@@ -20,36 +20,34 @@
 #ifndef UNICODE
 #error Win32 build requires a unicode build
 #endif
+#else
+#define _POSIX_SOURCE
+#define _LARGE_TIME_API
 #endif
 
 #include "ppsspp_config.h"
 
 #include "android/jni/app-android.h"
 
-#ifdef __MINGW32__
-#include <unistd.h>
-#ifndef _POSIX_THREAD_SAFE_FUNCTIONS
-#define _POSIX_THREAD_SAFE_FUNCTIONS 200112L
-#endif
-#endif
-
 #include <cstring>
 #include <ctime>
 #include <memory>
 
+#include <sys/types.h>
+
 #include "Common/Log.h"
 #include "Common/LogReporting.h"
 #include "Common/File/AndroidContentURI.h"
 #include "Common/File/FileUtil.h"
 #include "Common/StringUtils.h"
+#include "Common/TimeUtil.h"
 #include "Common/SysError.h"
+#include "Common/System/Request.h"
 
 #ifdef _WIN32
 #include "Common/CommonWindows.h"
-#include <Windows.h>
-#include <shlobj.h>		// for SHGetFolderPath
+#include <sys/utime.h>
 #include <shellapi.h>
-#include <commdlg.h>	// for GetSaveFileName
 #include <io.h>
 #include <direct.h>		// getcwd
 #if PPSSPP_PLATFORM(UWP)
@@ -63,6 +61,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <utime.h>
 #endif
 
 #if defined(__DragonFly__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__NetBSD__)
@@ -82,6 +81,14 @@
 
 #include <sys/stat.h>
 
+// NOTE: There's another one in DirListing.cpp.
+#ifdef _WIN32
+constexpr bool SIMULATE_SLOW_IO = false;
+#else
+constexpr bool SIMULATE_SLOW_IO = false;
+#endif
+constexpr bool LOG_IO = false;
+
 #ifndef S_ISDIR
 #define S_ISDIR(m)  (((m)&S_IFMT) == S_IFDIR)
 #endif
@@ -104,13 +111,19 @@
 namespace File {
 
 FILE *OpenCFile(const Path &path, const char *mode) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "OpenCFile %s, %s", path.c_str(), mode);
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(300, "slow-io-sim");
+	}
 	switch (path.Type()) {
 	case PathType::NATIVE:
 		break;
 	case PathType::CONTENT_URI:
 		// We're gonna need some error codes..
 		if (!strcmp(mode, "r") || !strcmp(mode, "rb") || !strcmp(mode, "rt")) {
-			INFO_LOG(COMMON, "Opening content file for read: '%s'", path.c_str());
+			INFO_LOG(Log::IO, "Opening content file for read: '%s'", path.c_str());
 			// Read, let's support this - easy one.
 			int descriptor = Android_OpenContentUriFd(path.ToString(), Android_OpenContentUriMode::READ);
 			if (descriptor < 0) {
@@ -119,22 +132,24 @@ FILE *OpenCFile(const Path &path, const char *mode) {
 			return fdopen(descriptor, "rb");
 		} else if (!strcmp(mode, "w") || !strcmp(mode, "wb") || !strcmp(mode, "wt") || !strcmp(mode, "at") || !strcmp(mode, "a")) {
 			// Need to be able to create the file here if it doesn't exist.
+			// NOTE: The existance check is important, otherwise Android will create a numbered file by the side!
+			// This is also a terrible possible data race, ugh. Anyway...
 			// Not exactly sure which abstractions are best, let's start simple.
 			if (!File::Exists(path)) {
-				INFO_LOG(COMMON, "OpenCFile(%s): Opening content file for write. Doesn't exist, creating empty and reopening.", path.c_str());
+				INFO_LOG(Log::IO, "OpenCFile(%s): Opening content file for write. Doesn't exist, creating empty and reopening.", path.c_str());
 				std::string name = path.GetFilename();
 				if (path.CanNavigateUp()) {
 					Path parent = path.NavigateUp();
 					if (Android_CreateFile(parent.ToString(), name) != StorageError::SUCCESS) {
-						WARN_LOG(COMMON, "Failed to create file '%s' in '%s'", name.c_str(), parent.c_str());
+						WARN_LOG(Log::IO, "Failed to create file '%s' in '%s'", name.c_str(), parent.c_str());
 						return nullptr;
 					}
 				} else {
-					INFO_LOG_REPORT_ONCE(openCFileFailedNavigateUp, COMMON, "Failed to navigate up to create file: %s", path.c_str());
+					INFO_LOG(Log::IO, "Failed to navigate up to create file: %s", path.c_str());
 					return nullptr;
 				}
 			} else {
-				INFO_LOG(COMMON, "OpenCFile(%s): Opening existing content file for write (truncating). Requested mode: '%s'", path.c_str(), mode);
+				INFO_LOG(Log::IO, "OpenCFile(%s): Opening existing content file for write (truncating). Requested mode: '%s'", path.c_str(), mode);
 			}
 
 			// TODO: Support append modes and stuff... For now let's go with the most common one.
@@ -146,7 +161,7 @@ FILE *OpenCFile(const Path &path, const char *mode) {
 			}
 			int descriptor = Android_OpenContentUriFd(path.ToString(), openMode);
 			if (descriptor < 0) {
-				INFO_LOG(COMMON, "Opening '%s' for write failed", path.ToString().c_str());
+				INFO_LOG(Log::IO, "Opening '%s' for write failed", path.ToString().c_str());
 				return nullptr;
 			}
 			FILE *f = fdopen(descriptor, fmode);
@@ -156,18 +171,18 @@ FILE *OpenCFile(const Path &path, const char *mode) {
 			}
 			return f;
 		} else {
-			ERROR_LOG(COMMON, "OpenCFile(%s): Mode not yet supported: %s", path.c_str(), mode);
+			ERROR_LOG(Log::IO, "OpenCFile(%s): Mode not yet supported: %s", path.c_str(), mode);
 			return nullptr;
 		}
 		break;
 	default:
-		ERROR_LOG(COMMON, "OpenCFile(%s): PathType not yet supported", path.c_str());
+		ERROR_LOG(Log::IO, "OpenCFile(%s): PathType not yet supported", path.c_str());
 		return nullptr;
 	}
 
 #if defined(_WIN32) && defined(UNICODE)
 #if PPSSPP_PLATFORM(UWP) && !defined(__LIBRETRO__)
-	// We shouldn't use _wfopen here, 
+	// We shouldn't use _wfopen here,
 	// this function is not allowed to read outside Local and Installation folders
 	// FileSystem (broadFileSystemAccess) doesn't apply on _wfopen
 	// if we have custom memory stick location _wfopen will return null
@@ -202,36 +217,47 @@ static std::string OpenFlagToString(OpenFlag flags) {
 }
 
 int OpenFD(const Path &path, OpenFlag flags) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "OpenFD %s, %d", path.c_str(), flags);
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(300, "slow-io-sim");
+	}
+
 	switch (path.Type()) {
 	case PathType::CONTENT_URI:
 		break;
 	default:
-		ERROR_LOG(COMMON, "OpenFD: Only supports Content URI paths. Not '%s' (%s)!", path.c_str(), OpenFlagToString(flags).c_str());
+		ERROR_LOG(Log::IO, "OpenFD: Only supports Content URI paths. Not '%s' (%s)!", path.c_str(), OpenFlagToString(flags).c_str());
 		// Not yet supported - use other paths.
 		return -1;
 	}
 
+	bool knownExists = false;
+
 	if (flags & OPEN_CREATE) {
 		if (!File::Exists(path)) {
-			INFO_LOG(COMMON, "OpenFD(%s): Creating file.", path.c_str());
+			INFO_LOG(Log::IO, "OpenFD(%s): Creating file.", path.c_str());
 			std::string name = path.GetFilename();
 			if (path.CanNavigateUp()) {
 				Path parent = path.NavigateUp();
 				if (Android_CreateFile(parent.ToString(), name) != StorageError::SUCCESS) {
-					WARN_LOG(COMMON, "OpenFD: Failed to create file '%s' in '%s'", name.c_str(), parent.c_str());
+					WARN_LOG(Log::IO, "OpenFD: Failed to create file '%s' in '%s'", name.c_str(), parent.c_str());
 					return -1;
 				}
+				knownExists = true;
 			} else {
-				INFO_LOG(COMMON, "Failed to navigate up to create file: %s", path.c_str());
+				INFO_LOG(Log::IO, "Failed to navigate up to create file: %s", path.c_str());
 				return -1;
 			}
 		} else {
-			INFO_LOG(COMMON, "OpenCFile(%s): Opening existing content file ('%s')", path.c_str(), OpenFlagToString(flags).c_str());
+			INFO_LOG(Log::IO, "OpenCFile(%s): Opening existing content file ('%s')", path.c_str(), OpenFlagToString(flags).c_str());
+			knownExists = true;
 		}
 	}
 
 	Android_OpenContentUriMode mode;
-	if (flags == OPEN_READ) {
+	if (flags == OPEN_READ) {  // Intentionally not a bitfield check.
 		mode = Android_OpenContentUriMode::READ;
 	} else if (flags & OPEN_WRITE) {
 		if (flags & OPEN_TRUNCATE) {
@@ -242,24 +268,33 @@ int OpenFD(const Path &path, OpenFlag flags) {
 		// TODO: Maybe better checking of additional flags here.
 	} else {
 		// TODO: Add support for more modes if possible.
-		ERROR_LOG_REPORT_ONCE(openFlagNotSupported, COMMON, "OpenFlag %s not yet supported", OpenFlagToString(flags).c_str());
+		ERROR_LOG_REPORT_ONCE(openFlagNotSupported, Log::IO, "OpenFlag %s not yet supported", OpenFlagToString(flags).c_str());
 		return -1;
 	}
 
-	INFO_LOG(COMMON, "Android_OpenContentUriFd: %s (%s)", path.c_str(), OpenFlagToString(flags).c_str());
+	INFO_LOG(Log::IO, "Android_OpenContentUriFd: %s (%s)", path.c_str(), OpenFlagToString(flags).c_str());
 	int descriptor = Android_OpenContentUriFd(path.ToString(), mode);
 	if (descriptor < 0) {
-		ERROR_LOG(COMMON, "Android_OpenContentUriFd failed: '%s'", path.c_str());
-	}
-
-	if (flags & OPEN_APPEND) {
+		// File probably just doesn't exist. No biggie.
+		if (knownExists) {
+			ERROR_LOG(Log::IO, "Android_OpenContentUriFd failed for existing file: '%s'", path.c_str());
+		} else {
+			INFO_LOG(Log::IO, "Android_OpenContentUriFd failed, probably doesn't exist: '%s'", path.c_str());
+		}
+	} else if (flags & OPEN_APPEND) {
 		// Simply seek to the end of the file to simulate append mode.
 		lseek(descriptor, 0, SEEK_END);
 	}
-
 	return descriptor;
 }
 
+void CloseFD(int fd) {
+#if PPSSPP_PLATFORM(ANDROID)
+	close(fd);
+#endif
+}
+
+
 #ifdef _WIN32
 static bool ResolvePathVista(const std::wstring &path, wchar_t *buf, DWORD bufSize) {
 	typedef DWORD(WINAPI *getFinalPathNameByHandleW_f)(HANDLE hFile, LPWSTR lpszFilePath, DWORD cchFilePath, DWORD dwFlags);
@@ -294,14 +329,22 @@ static bool ResolvePathVista(const std::wstring &path, wchar_t *buf, DWORD bufSi
 }
 #endif
 
-std::string ResolvePath(const std::string &path) {
+// Canonicalize the given path, resolving symlinks, relative paths, etc.
+std::string ResolvePath(std::string_view path) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "ResolvePath %.*s", (int)path.size(), path.data());
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+	}
+
 	if (startsWith(path, "http://") || startsWith(path, "https://")) {
-		return path;
+		return std::string(path);
 	}
 
 	if (Android_IsContentUri(path)) {
-		// Nothing to do?
-		return path;
+		// Nothing to do? We consider these to only have one canonical form.
+		return std::string(path);
 	}
 
 #ifdef _WIN32
@@ -340,11 +383,15 @@ std::string ResolvePath(const std::string &path) {
 	delete [] buf;
 	return output;
 
+#elif PPSSPP_PLATFORM(IOS)
+	// Resolving has wacky effects on documents paths.
+	return std::string(path);
 #else
 	std::unique_ptr<char[]> buf(new char[PATH_MAX + 32768]);
-	if (realpath(path.c_str(), buf.get()) == nullptr)
-		return path;
-	return buf.get();
+	std::string spath(path);
+	if (realpath(spath.c_str(), buf.get()) == nullptr)
+		return spath;
+	return std::string(buf.get());
 #endif
 }
 
@@ -375,11 +422,18 @@ uint64_t ComputeRecursiveDirectorySize(const Path &path) {
 }
 
 // Returns true if file filename exists. Will return true on directories.
-bool ExistsInDir(const Path &path, const std::string &filename) {
+bool ExistsInDir(const Path &path, std::string_view filename) {
 	return Exists(path / filename);
 }
 
 bool Exists(const Path &path) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "Exists %s", path.ToVisualString().c_str());
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(200, "slow-io-sim");
+	}
+
 	if (path.Type() == PathType::CONTENT_URI) {
 		return Android_FileExists(path.c_str());
 	}
@@ -404,21 +458,28 @@ bool Exists(const Path &path) {
 	SetErrorMode(OldMode);
 #endif
 	return true;
-#else
-	struct stat file_info;
+#else  // !WIN32
+	struct stat file_info{};
 	return stat(path.c_str(), &file_info) == 0;
 #endif
 }
 
 // Returns true if filename exists and is a directory
-bool IsDirectory(const Path &filename) {
-	switch (filename.Type()) {
+bool IsDirectory(const Path &path) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "IsDirectory %s", path.c_str());
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+	}
+
+	switch (path.Type()) {
 	case PathType::NATIVE:
 		break; // OK
 	case PathType::CONTENT_URI:
 	{
 		FileInfo info;
-		if (!Android_GetFileInfo(filename.ToString(), &info)) {
+		if (!Android_GetFileInfo(path.ToString(), &info)) {
 			return false;
 		}
 		return info.exists && info.isDirectory;
@@ -430,24 +491,24 @@ bool IsDirectory(const Path &filename) {
 #if defined(_WIN32)
 	WIN32_FILE_ATTRIBUTE_DATA data{};
 #if PPSSPP_PLATFORM(UWP)
-	if (!GetFileAttributesExFromAppW(filename.ToWString().c_str(), GetFileExInfoStandard, &data) || data.dwFileAttributes == INVALID_FILE_ATTRIBUTES) {
+	if (!GetFileAttributesExFromAppW(path.ToWString().c_str(), GetFileExInfoStandard, &data) || data.dwFileAttributes == INVALID_FILE_ATTRIBUTES) {
 #else
-	if (!GetFileAttributesEx(filename.ToWString().c_str(), GetFileExInfoStandard, &data) || data.dwFileAttributes == INVALID_FILE_ATTRIBUTES) {
+	if (!GetFileAttributesEx(path.ToWString().c_str(), GetFileExInfoStandard, &data) || data.dwFileAttributes == INVALID_FILE_ATTRIBUTES) {
 #endif
 		auto err = GetLastError();
 		if (err != ERROR_FILE_NOT_FOUND) {
-			WARN_LOG(COMMON, "GetFileAttributes failed on %s: %08x %s", filename.ToVisualString().c_str(), (uint32_t)err, GetStringErrorMsg(err).c_str());
+			WARN_LOG(Log::IO, "GetFileAttributes failed on %s: %08x %s", path.ToVisualString().c_str(), (uint32_t)err, GetStringErrorMsg(err).c_str());
 		}
 		return false;
 	}
 	DWORD result = data.dwFileAttributes;
 	return (result & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY;
 #else
-	std::string copy = filename.ToString();
-	struct stat file_info;
+	std::string copy = path.ToString();
+	struct stat file_info{};
 	int result = stat(copy.c_str(), &file_info);
 	if (result < 0) {
-		WARN_LOG(COMMON, "IsDirectory: stat failed on %s: %s", copy.c_str(), GetLastErrorMsg().c_str());
+		WARN_LOG(Log::IO, "IsDirectory: stat failed on %s: %s", copy.c_str(), GetLastErrorMsg().c_str());
 		return false;
 	}
 	return S_ISDIR(file_info.st_mode);
@@ -457,6 +518,9 @@ bool IsDirectory(const Path &filename) {
 // Deletes a given filename, return true on success
 // Doesn't supports deleting a directory
 bool Delete(const Path &filename) {
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(200, "slow-io-sim");
+	}
 	switch (filename.Type()) {
 	case PathType::NATIVE:
 		break; // OK
@@ -466,46 +530,49 @@ bool Delete(const Path &filename) {
 		return false;
 	}
 
-	INFO_LOG(COMMON, "Delete: file %s", filename.c_str());
-
-	// Return true because we care about the file no 
+	// Return true because we care about the file no
 	// being there, not the actual delete.
 	if (!Exists(filename)) {
-		WARN_LOG(COMMON, "Delete: '%s' already does not exist", filename.c_str());
+		WARN_LOG(Log::IO, "Delete: '%s' already does not exist", filename.c_str());
 		return true;
 	}
 
 	// We can't delete a directory
 	if (IsDirectory(filename)) {
-		WARN_LOG(COMMON, "Delete failed: '%s' is a directory", filename.c_str());
+		WARN_LOG(Log::IO, "Delete failed: '%s' is a directory", filename.c_str());
 		return false;
 	}
 
 #ifdef _WIN32
 #if PPSSPP_PLATFORM(UWP)
 	if (!DeleteFileFromAppW(filename.ToWString().c_str())) {
-		WARN_LOG(COMMON, "Delete: DeleteFile failed on %s: %s", filename.c_str(), GetLastErrorMsg().c_str());
+		WARN_LOG(Log::IO, "Delete: DeleteFile failed on %s: %s", filename.c_str(), GetLastErrorMsg().c_str());
 		return false;
 	}
 #else
 	if (!DeleteFile(filename.ToWString().c_str())) {
-		WARN_LOG(COMMON, "Delete: DeleteFile failed on %s: %s", filename.c_str(), GetLastErrorMsg().c_str());
+		WARN_LOG(Log::IO, "Delete: DeleteFile failed on %s: %s", filename.c_str(), GetLastErrorMsg().c_str());
 		return false;
 	}
 #endif
 #else
 	if (unlink(filename.c_str()) == -1) {
-		WARN_LOG(COMMON, "Delete: unlink failed on %s: %s", 
+		WARN_LOG(Log::IO, "Delete: unlink failed on %s: %s",
 				 filename.c_str(), GetLastErrorMsg().c_str());
 		return false;
 	}
 #endif
 
+	INFO_LOG(Log::IO, "Delete: file %s was deleted.", filename.c_str());
 	return true;
 }
 
 // Returns true if successful, or path already exists.
 bool CreateDir(const Path &path) {
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+		INFO_LOG(Log::IO, "CreateDir %s", path.c_str());
+	}
 	switch (path.Type()) {
 	case PathType::NATIVE:
 		break; // OK
@@ -522,11 +589,11 @@ bool CreateDir(const Path &path) {
 		AndroidContentURI uri(path.ToString());
 		std::string newDirName = uri.GetLastPart();
 		if (uri.NavigateUp()) {
-			INFO_LOG(COMMON, "Calling Android_CreateDirectory(%s, %s)", uri.ToString().c_str(), newDirName.c_str());
+			INFO_LOG(Log::IO, "Calling Android_CreateDirectory(%s, %s)", uri.ToString().c_str(), newDirName.c_str());
 			return Android_CreateDirectory(uri.ToString(), newDirName) == StorageError::SUCCESS;
 		} else {
 			// Bad path - can't create this directory.
-			WARN_LOG(COMMON, "CreateDir failed: '%s'", path.c_str());
+			WARN_LOG(Log::IO, "CreateDir failed: '%s'", path.c_str());
 			return false;
 		}
 		break;
@@ -535,7 +602,7 @@ bool CreateDir(const Path &path) {
 		return false;
 	}
 
-	DEBUG_LOG(COMMON, "CreateDir('%s')", path.c_str());
+	DEBUG_LOG(Log::IO, "CreateDir('%s')", path.c_str());
 #ifdef _WIN32
 #if PPSSPP_PLATFORM(UWP)
 	if (CreateDirectoryFromAppW(path.ToWString().c_str(), NULL))
@@ -544,13 +611,13 @@ bool CreateDir(const Path &path) {
 	if (::CreateDirectory(path.ToWString().c_str(), NULL))
 		return true;
 #endif
-	
+
 	DWORD error = GetLastError();
 	if (error == ERROR_ALREADY_EXISTS) {
-		WARN_LOG(COMMON, "CreateDir: CreateDirectory failed on %s: already exists", path.c_str());
+		DEBUG_LOG(Log::IO, "CreateDir: CreateDirectory failed on %s: already exists", path.c_str());
 		return true;
 	}
-	ERROR_LOG(COMMON, "CreateDir: CreateDirectory failed on %s: %08x %s", path.c_str(), (uint32_t)error, GetStringErrorMsg(error).c_str());
+	ERROR_LOG(Log::IO, "CreateDir: CreateDirectory failed on %s: %08x %s", path.c_str(), (uint32_t)error, GetStringErrorMsg(error).c_str());
 	return false;
 #else
 	if (mkdir(path.ToString().c_str(), 0755) == 0) {
@@ -559,11 +626,11 @@ bool CreateDir(const Path &path) {
 
 	int err = errno;
 	if (err == EEXIST) {
-		WARN_LOG(COMMON, "CreateDir: mkdir failed on %s: already exists", path.c_str());
+		DEBUG_LOG(Log::IO, "CreateDir: mkdir failed on %s: already exists", path.c_str());
 		return true;
 	}
 
-	ERROR_LOG(COMMON, "CreateDir: mkdir failed on %s: %s", path.c_str(), strerror(err));
+	ERROR_LOG(Log::IO, "CreateDir: mkdir failed on %s: %s", path.c_str(), strerror(err));
 	return false;
 #endif
 }
@@ -571,7 +638,7 @@ bool CreateDir(const Path &path) {
 // Creates the full path of fullPath returns true on success
 bool CreateFullPath(const Path &path) {
 	if (File::Exists(path)) {
-		DEBUG_LOG(COMMON, "CreateFullPath: path exists %s", path.c_str());
+		DEBUG_LOG(Log::IO, "CreateFullPath: path exists %s", path.ToVisualString().c_str());
 		return true;
 	}
 
@@ -580,7 +647,7 @@ bool CreateFullPath(const Path &path) {
 	case PathType::CONTENT_URI:
 		break; // OK
 	default:
-		ERROR_LOG(COMMON, "CreateFullPath(%s): Not yet supported", path.c_str());
+		ERROR_LOG(Log::IO, "CreateFullPath(%s): Not yet supported", path.ToVisualString().c_str());
 		return false;
 	}
 
@@ -594,27 +661,34 @@ bool CreateFullPath(const Path &path) {
 	}
 
 	std::vector<std::string_view> parts;
-	SplitString(diff, '/', parts);
+	if (!diff.empty()) {
+		SplitString(diff, '/', parts);
+	}
 
 	// Probably not necessary sanity check, ported from the old code.
 	if (parts.size() > 100) {
-		ERROR_LOG(COMMON, "CreateFullPath: directory structure too deep");
+		ERROR_LOG(Log::IO, "CreateFullPath: directory structure too deep");
 		return false;
 	}
 
 	Path curPath = root;
 	for (auto part : parts) {
 		curPath /= part;
-		if (!File::Exists(curPath)) {
-			File::CreateDir(curPath);
-		}
+		File::CreateDir(curPath);
 	}
 
 	return true;
 }
 
-// renames file srcFilename to destFilename, returns true on success 
+// renames file srcFilename to destFilename, returns true on success
 bool Rename(const Path &srcFilename, const Path &destFilename) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "Rename %s -> %s", srcFilename.c_str(), destFilename.c_str());
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+	}
+
 	if (srcFilename.Type() != destFilename.Type()) {
 		// Impossible. You're gonna need to make a copy, and delete the original. Not the responsibility
 		// of Rename.
@@ -630,16 +704,16 @@ bool Rename(const Path &srcFilename, const Path &destFilename) {
 		// Content URI: Can only rename if in the same folder.
 		// TODO: Fallback to move + rename? Or do we even care about that use case? We have MoveIfFast for such tricks.
 		if (srcFilename.GetDirectory() != destFilename.GetDirectory()) {
-			INFO_LOG(COMMON, "Content URI rename: Directories not matching, failing. %s --> %s", srcFilename.c_str(), destFilename.c_str());
+			INFO_LOG(Log::IO, "Content URI rename: Directories not matching, failing. %s --> %s", srcFilename.c_str(), destFilename.c_str());
 			return false;
 		}
-		INFO_LOG(COMMON, "Content URI rename: %s --> %s", srcFilename.c_str(), destFilename.c_str());
+		INFO_LOG(Log::IO, "Content URI rename: %s --> %s", srcFilename.c_str(), destFilename.c_str());
 		return Android_RenameFileTo(srcFilename.ToString(), destFilename.GetFilename()) == StorageError::SUCCESS;
 	default:
 		return false;
 	}
 
-	INFO_LOG(COMMON, "Rename: %s --> %s", srcFilename.c_str(), destFilename.c_str());
+	INFO_LOG(Log::IO, "Rename: %s --> %s", srcFilename.c_str(), destFilename.c_str());
 
 #if defined(_WIN32) && defined(UNICODE)
 #if PPSSPP_PLATFORM(UWP)
@@ -656,13 +730,19 @@ bool Rename(const Path &srcFilename, const Path &destFilename) {
 		return true;
 #endif
 
-	ERROR_LOG(COMMON, "Rename: failed %s --> %s: %s", 
+	ERROR_LOG(Log::IO, "Rename: failed %s --> %s: %s",
 			  srcFilename.c_str(), destFilename.c_str(), GetLastErrorMsg().c_str());
 	return false;
 }
 
-// copies file srcFilename to destFilename, returns true on success 
+// copies file srcFilename to destFilename, returns true on success
 bool Copy(const Path &srcFilename, const Path &destFilename) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "Copy %s -> %s", srcFilename.c_str(), destFilename.c_str());
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+	}
 	switch (srcFilename.Type()) {
 	case PathType::NATIVE:
 		break; // OK
@@ -673,6 +753,7 @@ bool Copy(const Path &srcFilename, const Path &destFilename) {
 			if (Android_CopyFile(srcFilename.ToString(), destParent.ToString()) == StorageError::SUCCESS) {
 				return true;
 			}
+			INFO_LOG(Log::IO, "Android_CopyFile failed, falling back.");
 			// Else fall through, and try using file I/O.
 		}
 		break;
@@ -680,7 +761,7 @@ bool Copy(const Path &srcFilename, const Path &destFilename) {
 		return false;
 	}
 
-	INFO_LOG(COMMON, "Copy: %s --> %s", srcFilename.c_str(), destFilename.c_str());
+	INFO_LOG(Log::IO, "Copy by OpenCFile: %s --> %s", srcFilename.c_str(), destFilename.c_str());
 #ifdef _WIN32
 #if PPSSPP_PLATFORM(UWP)
 	if (CopyFileFromAppW(srcFilename.ToWString().c_str(), destFilename.ToWString().c_str(), FALSE))
@@ -689,10 +770,10 @@ bool Copy(const Path &srcFilename, const Path &destFilename) {
 	if (CopyFile(srcFilename.ToWString().c_str(), destFilename.ToWString().c_str(), FALSE))
 		return true;
 #endif
-	ERROR_LOG(COMMON, "Copy: failed %s --> %s: %s", 
+	ERROR_LOG(Log::IO, "Copy: failed %s --> %s: %s",
 			srcFilename.c_str(), destFilename.c_str(), GetLastErrorMsg().c_str());
 	return false;
-#else
+#else  // Non-Win32
 
 	// buffer size
 #define BSIZE 16384
@@ -702,7 +783,7 @@ bool Copy(const Path &srcFilename, const Path &destFilename) {
 	// Open input file
 	FILE *input = OpenCFile(srcFilename, "rb");
 	if (!input) {
-		ERROR_LOG(COMMON, "Copy: input failed %s --> %s: %s", 
+		ERROR_LOG(Log::IO, "Copy: input failed %s --> %s: %s",
 				srcFilename.c_str(), destFilename.c_str(), GetLastErrorMsg().c_str());
 		return false;
 	}
@@ -711,22 +792,24 @@ bool Copy(const Path &srcFilename, const Path &destFilename) {
 	FILE *output = OpenCFile(destFilename, "wb");
 	if (!output) {
 		fclose(input);
-		ERROR_LOG(COMMON, "Copy: output failed %s --> %s: %s", 
+		ERROR_LOG(Log::IO, "Copy: output failed %s --> %s: %s",
 				srcFilename.c_str(), destFilename.c_str(), GetLastErrorMsg().c_str());
 		return false;
 	}
 
+	int bytesWritten = 0;
+
 	// copy loop
 	while (!feof(input)) {
 		// read input
 		int rnum = fread(buffer, sizeof(char), BSIZE, input);
 		if (rnum != BSIZE) {
 			if (ferror(input) != 0) {
-				ERROR_LOG(COMMON, 
-						"Copy: failed reading from source, %s --> %s: %s", 
+				ERROR_LOG(Log::IO,
+						"Copy: failed reading from source, %s --> %s: %s",
 						srcFilename.c_str(), destFilename.c_str(), GetLastErrorMsg().c_str());
 				fclose(input);
-				fclose(output);		
+				fclose(output);
 				return false;
 			}
 		}
@@ -734,14 +817,21 @@ bool Copy(const Path &srcFilename, const Path &destFilename) {
 		// write output
 		int wnum = fwrite(buffer, sizeof(char), rnum, output);
 		if (wnum != rnum) {
-			ERROR_LOG(COMMON, 
-					"Copy: failed writing to output, %s --> %s: %s", 
+			ERROR_LOG(Log::IO,
+					"Copy: failed writing to output, %s --> %s: %s",
 					srcFilename.c_str(), destFilename.c_str(), GetLastErrorMsg().c_str());
 			fclose(input);
-			fclose(output);				
+			fclose(output);
 			return false;
 		}
+
+		bytesWritten += wnum;
 	}
+
+	if (bytesWritten == 0) {
+		WARN_LOG(Log::IO, "Copy: No bytes written (must mean that input was empty)");
+	}
+
 	// close flushes
 	fclose(input);
 	fclose(output);
@@ -751,6 +841,10 @@ bool Copy(const Path &srcFilename, const Path &destFilename) {
 
 // Will overwrite the target.
 bool Move(const Path &srcFilename, const Path &destFilename) {
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+		INFO_LOG(Log::IO, "Move %s -> %s", srcFilename.c_str(), destFilename.c_str());
+	}
 	bool fast = MoveIfFast(srcFilename, destFilename);
 	if (fast) {
 		return true;
@@ -789,6 +883,12 @@ bool MoveIfFast(const Path &srcFilename, const Path &destFilename) {
 // Returns the size of file (64bit)
 // TODO: Add a way to return an error.
 uint64_t GetFileSize(const Path &filename) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "GetFileSize %s", filename.c_str());
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+	}
 	switch (filename.Type()) {
 	case PathType::NATIVE:
 		break; // OK
@@ -826,14 +926,14 @@ uint64_t GetFileSize(const Path &filename) {
 	int result = stat64(filename.c_str(), &file_info);
 #endif
 	if (result != 0) {
-		WARN_LOG(COMMON, "GetSize: failed %s: No such file", filename.ToVisualString().c_str());
+		WARN_LOG(Log::IO, "GetSize: failed %s: No such file", filename.ToVisualString().c_str());
 		return 0;
 	}
 	if (S_ISDIR(file_info.st_mode)) {
-		WARN_LOG(COMMON, "GetSize: failed %s: is a directory", filename.ToVisualString().c_str());
+		WARN_LOG(Log::IO, "GetSize: failed %s: is a directory", filename.ToVisualString().c_str());
 		return 0;
 	}
-	DEBUG_LOG(COMMON, "GetSize: %s: %lld", filename.ToVisualString().c_str(), (long long)file_info.st_size);
+	DEBUG_LOG(Log::IO, "GetSize: %s: %lld", filename.ToVisualString().c_str(), (long long)file_info.st_size);
 	return file_info.st_size;
 #endif
 }
@@ -880,12 +980,12 @@ uint64_t GetFileSize(FILE *f) {
 #endif
 }
 
-// creates an empty file filename, returns true on success 
+// creates an empty file filename, returns true on success
 bool CreateEmptyFile(const Path &filename) {
-	INFO_LOG(COMMON, "CreateEmptyFile: %s", filename.c_str()); 
+	INFO_LOG(Log::IO, "CreateEmptyFile: %s", filename.c_str());
 	FILE *pFile = OpenCFile(filename, "wb");
 	if (!pFile) {
-		ERROR_LOG(COMMON, "CreateEmptyFile: failed to create '%s': %s", filename.c_str(), GetLastErrorMsg().c_str());
+		ERROR_LOG(Log::IO, "CreateEmptyFile: failed to create '%s': %s", filename.c_str(), GetLastErrorMsg().c_str());
 		return false;
 	}
 	fclose(pFile);
@@ -895,6 +995,12 @@ bool CreateEmptyFile(const Path &filename) {
 // Deletes an empty directory, returns true on success
 // WARNING: On Android with content URIs, it will delete recursively!
 bool DeleteDir(const Path &path) {
+	if (LOG_IO) {
+		INFO_LOG(Log::IO, "DeleteDir %s", path.c_str());
+	}
+	if (SIMULATE_SLOW_IO) {
+		sleep_ms(100, "slow-io-sim");
+	}
 	switch (path.Type()) {
 	case PathType::NATIVE:
 		break; // OK
@@ -903,11 +1009,11 @@ bool DeleteDir(const Path &path) {
 	default:
 		return false;
 	}
-	INFO_LOG(COMMON, "DeleteDir: directory %s", path.c_str());
+	INFO_LOG(Log::IO, "DeleteDir: directory %s", path.c_str());
 
 	// check if a directory
 	if (!File::IsDirectory(path)) {
-		ERROR_LOG(COMMON, "DeleteDir: Not a directory %s", path.c_str());
+		ERROR_LOG(Log::IO, "DeleteDir: Not a directory %s", path.c_str());
 		return false;
 	}
 
@@ -923,7 +1029,7 @@ bool DeleteDir(const Path &path) {
 	if (rmdir(path.c_str()) == 0)
 		return true;
 #endif
-	ERROR_LOG(COMMON, "DeleteDir: %s: %s", path.c_str(), GetLastErrorMsg().c_str());
+	ERROR_LOG(Log::IO, "DeleteDir: %s: %s", path.c_str(), GetLastErrorMsg().c_str());
 
 	return false;
 }
@@ -937,7 +1043,7 @@ bool DeleteDirRecursively(const Path &path) {
 		// We make use of the dangerous auto-recursive property of Android_RemoveFile.
 		return Android_RemoveFile(path.ToString()) == StorageError::SUCCESS;
 	default:
-		ERROR_LOG(COMMON, "DeleteDirRecursively: Path type not supported");
+		ERROR_LOG(Log::IO, "DeleteDirRecursively: Path type not supported");
 		return false;
 	}
 
@@ -958,7 +1064,7 @@ bool OpenFileInEditor(const Path &fileName) {
 	case PathType::NATIVE:
 		break;  // OK
 	default:
-		ERROR_LOG(COMMON, "OpenFileInEditor(%s): Path type not supported", fileName.c_str());
+		ERROR_LOG(Log::IO, "OpenFileInEditor(%s): Path type not supported", fileName.c_str());
 		return false;
 	}
 
@@ -976,10 +1082,10 @@ bool OpenFileInEditor(const Path &fileName) {
 	iniFile = "xdg-open ";
 #endif
 	iniFile.append(fileName.ToString());
-	NOTICE_LOG(BOOT, "Launching %s", iniFile.c_str());
+	NOTICE_LOG(Log::Boot, "Launching %s", iniFile.c_str());
 	int retval = system(iniFile.c_str());
 	if (retval != 0) {
-		ERROR_LOG(COMMON, "Failed to launch ini file");
+		ERROR_LOG(Log::IO, "Failed to launch ini file");
 	}
 #endif
 	return true;
@@ -1115,7 +1221,7 @@ bool IOFile::Seek(int64_t off, int origin)
 }
 
 uint64_t IOFile::Tell()
-{	
+{
 	if (IsOpen())
 		return ftello(m_file);
 	else
@@ -1210,7 +1316,7 @@ uint8_t *ReadLocalFile(const Path &filename, size_t *size) {
 	return contents;
 }
 
-bool WriteStringToFile(bool text_file, const std::string &str, const Path &filename) {
+bool WriteStringToFile(bool text_file, std::string_view str, const Path &filename) {
 	FILE *f = File::OpenCFile(filename, text_file ? "w" : "wb");
 	if (!f)
 		return false;
@@ -1237,4 +1343,42 @@ bool WriteDataToFile(bool text_file, const void* data, size_t size, const Path &
 	return true;
 }
 
+void ChangeMTime(const Path &path, time_t mtime) {
+	if (path.Type() == PathType::CONTENT_URI) {
+		// No clue what to do here. There doesn't seem to be a way.
+		return;
+	}
+
+#ifdef _WIN32
+	_utimbuf buf{};
+	buf.actime = mtime;
+	buf.modtime = mtime;
+	_utime(path.c_str(), &buf);
+#else
+	utimbuf buf{};
+	buf.actime = mtime;
+	buf.modtime = mtime;
+	utime(path.c_str(), &buf);
+#endif
+}
+
+bool IsProbablyInDownloadsFolder(const Path &filename) {
+	INFO_LOG(Log::IO, "IsProbablyInDownloadsFolder: Looking at %s (%s)...", filename.c_str(), filename.ToVisualString().c_str());
+	switch (filename.Type()) {
+	case PathType::CONTENT_URI:
+	{
+		AndroidContentURI uri(filename.ToString());
+		INFO_LOG(Log::IO, "Content URI provider: %s", uri.Provider().c_str());
+		if (containsNoCase(uri.Provider(), "download")) {
+			// like com.android.providers.downloads.documents
+			return true;
+		}
+		break;
+	}
+	default:
+		break;
+	}
+	return filename.FilePathContainsNoCase("download");
+}
+
 }  // namespace File
