@@ -1,5 +1,4 @@
 #include "ppsspp_config.h"
-#include <algorithm>
 
 #include "Common/GPU/OpenGL/GLCommon.h"
 #include "Common/GPU/OpenGL/GLDebugLog.h"
@@ -107,19 +106,6 @@ static std::string GetInfoLog(GLuint name, Getiv getiv, GetLog getLog) {
 	return infoLog;
 }
 
-static int GetStereoBufferIndex(const char *uniformName) {
-	if (!uniformName) return -1;
-	else if (strcmp(uniformName, "u_view") == 0) return 0;
-	else if (strcmp(uniformName, "u_proj_lens") == 0) return 1;
-	else return -1;
-}
-
-static std::string GetStereoBufferLayout(const char *uniformName) {
-	if (strcmp(uniformName, "u_view") == 0) return "ViewMatrices";
-	else if (strcmp(uniformName, "u_proj_lens") == 0) return "ProjectionMatrix";
-	else return "undefined";
-}
-
 void GLQueueRunner::RunInitSteps(const FastVec<GLRInitStep> &steps, bool skipGLCalls) {
 	if (skipGLCalls) {
 		// Some bookkeeping still needs to be done.
@@ -143,12 +129,12 @@ void GLQueueRunner::RunInitSteps(const FastVec<GLRInitStep> &steps, bool skipGLC
 			}
 			case GLRInitStepType::CREATE_PROGRAM:
 			{
-				WARN_LOG(G3D, "CREATE_PROGRAM found with skipGLCalls, not good");
+				WARN_LOG(Log::G3D, "CREATE_PROGRAM found with skipGLCalls, not good");
 				break;
 			}
 			case GLRInitStepType::CREATE_SHADER:
 			{
-				WARN_LOG(G3D, "CREATE_SHADER found with skipGLCalls, not good");
+				WARN_LOG(Log::G3D, "CREATE_SHADER found with skipGLCalls, not good");
 				break;
 			}
 			default:
@@ -212,7 +198,7 @@ void GLQueueRunner::RunInitSteps(const FastVec<GLRInitStep> &steps, bool skipGLC
 				glAttachShader(program->program, step.create_program.shaders[j]->shader);
 			}
 
-			for (auto iter : program->semantics_) {
+			for (const auto &iter : program->semantics_) {
 				glBindAttribLocation(program->program, iter.location, iter.attrib);
 			}
 
@@ -250,11 +236,11 @@ void GLQueueRunner::RunInitSteps(const FastVec<GLRInitStep> &steps, bool skipGLC
 				if (!anyFailed)
 					Reporting::ReportMessage("Error in shader program link: info: %s\nfs: %s\n%s\nvs: %s\n%s", infoLog.c_str(), fsDesc.c_str(), fsCode, vsDesc.c_str(), vsCode);
 
-				ERROR_LOG(G3D, "Could not link program:\n %s", infoLog.c_str());
-				ERROR_LOG(G3D, "VS desc:\n%s", vsDesc.c_str());
-				ERROR_LOG(G3D, "FS desc:\n%s", fsDesc.c_str());
-				ERROR_LOG(G3D, "VS:\n%s\n", LineNumberString(vsCode).c_str());
-				ERROR_LOG(G3D, "FS:\n%s\n", LineNumberString(fsCode).c_str());
+				ERROR_LOG(Log::G3D, "Could not link program:\n %s", infoLog.c_str());
+				ERROR_LOG(Log::G3D, "VS desc:\n%s", vsDesc.c_str());
+				ERROR_LOG(Log::G3D, "FS desc:\n%s", fsDesc.c_str());
+				ERROR_LOG(Log::G3D, "VS:\n%s\n", LineNumberString(vsCode).c_str());
+				ERROR_LOG(Log::G3D, "FS:\n%s\n", LineNumberString(fsCode).c_str());
 
 #ifdef _WIN32
 				OutputDebugStringUTF8(infoLog.c_str());
@@ -274,28 +260,10 @@ void GLQueueRunner::RunInitSteps(const FastVec<GLRInitStep> &steps, bool skipGLC
 				auto &query = program->queries_[j];
 				_dbg_assert_(query.name);
 
-				int location = -1;
-				if (IsVREnabled() && IsMultiviewSupported()) {
-					int index = GetStereoBufferIndex(query.name);
-					if (index >= 0) {
-						std::string layout = GetStereoBufferLayout(query.name);
-						glUniformBlockBinding(program->program, glGetUniformBlockIndex(program->program, layout.c_str()), index);
-
-						GLuint buffer = 0;
-						glGenBuffers(1, &buffer);
-						glBindBuffer(GL_UNIFORM_BUFFER, buffer);
-						glBufferData(GL_UNIFORM_BUFFER,2 * 16 * sizeof(float),NULL, GL_STATIC_DRAW);
-						glBindBuffer(GL_UNIFORM_BUFFER, 0);
-						location = buffer;
-					} else {
-						location = glGetUniformLocation(program->program, query.name);
-					}
-				} else {
-					location = glGetUniformLocation(program->program, query.name);
-				}
+				int location = glGetUniformLocation(program->program, query.name);
 
 				if (location < 0 && query.required) {
-					WARN_LOG(G3D, "Required uniform query for '%s' failed", query.name);
+					WARN_LOG(Log::G3D, "Required uniform query for '%s' failed", query.name);
 				}
 				*query.dest = location;
 			}
@@ -336,8 +304,8 @@ void GLQueueRunner::RunInitSteps(const FastVec<GLRInitStep> &steps, bool skipGLC
 					LineNumberString(code).c_str());
 				std::vector<std::string_view> lines;
 				SplitString(errorString, '\n', lines);
-				for (auto line : lines) {
-					ERROR_LOG(G3D, "%.*s", (int)line.size(), line.data());
+				for (const auto &line : lines) {
+					ERROR_LOG(Log::G3D, "%.*s", (int)line.size(), line.data());
 				}
 				if (errorCallback_) {
 					std::string desc = StringFromFormat("Shader compilation failed: %s", step.create_shader.stage == GL_VERTEX_SHADER ? "vertex" : "fragment");
@@ -462,12 +430,12 @@ void GLQueueRunner::RunInitSteps(const FastVec<GLRInitStep> &steps, bool skipGLC
 		// Calling glGetError() isn't great, but at the end of init, only after creating textures, shouldn't be too bad...
 		GLenum err = glGetError();
 		if (err == GL_OUT_OF_MEMORY) {
-			WARN_LOG_REPORT(G3D, "GL ran out of GPU memory; switching to low memory mode");
+			WARN_LOG(Log::G3D, "GL ran out of GPU memory; switching to low memory mode");
 			sawOutOfMemory_ = true;
 		} else if (err != GL_NO_ERROR) {
 			// We checked the err anyway, might as well log if there is one.
 			std::string errorString = GLEnumToString(err);
-			WARN_LOG(G3D, "Got an error after init: %08x (%s)", err, errorString.c_str());
+			WARN_LOG(Log::G3D, "Got an error after init: %08x (%s)", err, errorString.c_str());
 			if (errorCallback_) {
 				errorCallback_("GL frame init error", errorString.c_str(), errorCallbackUserData_);
 			}
@@ -525,7 +493,7 @@ void GLQueueRunner::InitCreateFramebuffer(const GLRInitStep &step) {
 
 retry_depth:
 	if (!fbo->z_stencil_) {
-		INFO_LOG(G3D, "Creating %d x %d FBO using no depth", fbo->width, fbo->height);
+		INFO_LOG(Log::G3D, "Creating %d x %d FBO using no depth", fbo->width, fbo->height);
 
 		fbo->z_stencil_buffer = 0;
 		fbo->stencil_buffer = 0;
@@ -538,7 +506,7 @@ retry_depth:
 		glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_STENCIL_ATTACHMENT, GL_RENDERBUFFER, 0);
 	} else if (gl_extensions.IsGLES) {
 		if (gl_extensions.OES_packed_depth_stencil && (gl_extensions.OES_depth_texture || gl_extensions.GLES3)) {
-			INFO_LOG(G3D, "Creating %d x %d FBO using DEPTH24_STENCIL8 texture", fbo->width, fbo->height);
+			INFO_LOG(Log::G3D, "Creating %d x %d FBO using DEPTH24_STENCIL8 texture", fbo->width, fbo->height);
 			fbo->z_stencil_buffer = 0;
 			fbo->stencil_buffer = 0;
 			fbo->z_buffer = 0;
@@ -559,7 +527,7 @@ retry_depth:
 				glFramebufferTexture2D(GL_FRAMEBUFFER, GL_STENCIL_ATTACHMENT, GL_TEXTURE_2D, fbo->z_stencil_texture.texture, 0);
 			}
 		} else if (gl_extensions.OES_packed_depth_stencil) {
-			INFO_LOG(G3D, "Creating %d x %d FBO using DEPTH24_STENCIL8", fbo->width, fbo->height);
+			INFO_LOG(Log::G3D, "Creating %d x %d FBO using DEPTH24_STENCIL8", fbo->width, fbo->height);
 			// Standard method
 			fbo->stencil_buffer = 0;
 			fbo->z_buffer = 0;
@@ -574,7 +542,7 @@ retry_depth:
 			glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, fbo->z_stencil_buffer);
 			glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_STENCIL_ATTACHMENT, GL_RENDERBUFFER, fbo->z_stencil_buffer);
 		} else {
-			INFO_LOG(G3D, "Creating %d x %d FBO using separate stencil", fbo->width, fbo->height);
+			INFO_LOG(Log::G3D, "Creating %d x %d FBO using separate stencil", fbo->width, fbo->height);
 			// TEGRA
 			fbo->z_stencil_buffer = 0;
 			// 16/24-bit Z, separate 8-bit stencil
@@ -595,7 +563,7 @@ retry_depth:
 			glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_STENCIL_ATTACHMENT, GL_RENDERBUFFER, fbo->stencil_buffer);
 		}
 	} else if (gl_extensions.VersionGEThan(3, 0)) {
-		INFO_LOG(G3D, "Creating %d x %d FBO using DEPTH24_STENCIL8 texture", fbo->width, fbo->height);
+		INFO_LOG(Log::G3D, "Creating %d x %d FBO using DEPTH24_STENCIL8 texture", fbo->width, fbo->height);
 		fbo->z_stencil_buffer = 0;
 		fbo->stencil_buffer = 0;
 		fbo->z_buffer = 0;
@@ -631,13 +599,13 @@ retry_depth:
 
 	switch (status) {
 	case GL_FRAMEBUFFER_COMPLETE:
-		// INFO_LOG(G3D, "Framebuffer verified complete.");
+		// INFO_LOG(Log::G3D, "Framebuffer verified complete.");
 		break;
 	case GL_FRAMEBUFFER_UNSUPPORTED:
-		ERROR_LOG(G3D, "GL_FRAMEBUFFER_UNSUPPORTED");
+		ERROR_LOG(Log::G3D, "GL_FRAMEBUFFER_UNSUPPORTED");
 		break;
 	case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT:
-		ERROR_LOG(G3D, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT");
+		ERROR_LOG(Log::G3D, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT");
 		break;
 	default:
 		_assert_msg_(false, "Other framebuffer error: %d", status);
@@ -778,7 +746,7 @@ void GLQueueRunner::PerformBlit(const GLRStep &step) {
 		CHECK_GL_ERROR_IF_DEBUG();
 #endif // defined(USING_GLES2) && defined(__ANDROID__)
 	} else {
-		ERROR_LOG(G3D, "GLQueueRunner: Tried to blit without the capability");
+		ERROR_LOG(Log::G3D, "GLQueueRunner: Tried to blit without the capability");
 	}
 }
 
@@ -1119,36 +1087,22 @@ void GLQueueRunner::PerformRenderPass(const GLRStep &step, bool first, bool last
 		case GLRRenderCommand::UNIFORMSTEREOMATRIX:
 		{
 			_dbg_assert_(curProgram);
-			if (IsMultiviewSupported()) {
-				int layout = GetStereoBufferIndex(c.uniformStereoMatrix4.name);
-				if (layout >= 0) {
-					int size = 2 * 16 * sizeof(float);
-					glBindBufferBase(GL_UNIFORM_BUFFER, layout, *c.uniformStereoMatrix4.loc);
-					glBindBuffer(GL_UNIFORM_BUFFER, *c.uniformStereoMatrix4.loc);
-					void *matrices = glMapBufferRange(GL_UNIFORM_BUFFER, 0, size, GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT);
-					memcpy(matrices, c.uniformStereoMatrix4.mData, size);
-					glUnmapBuffer(GL_UNIFORM_BUFFER);
-					glBindBuffer(GL_UNIFORM_BUFFER, 0);
-				}
-				delete[] c.uniformStereoMatrix4.mData;  // We only playback once.
-			} else {
-				int loc = c.uniformStereoMatrix4.loc ? *c.uniformStereoMatrix4.loc : -1;
-				if (c.uniformStereoMatrix4.name) {
-					loc = curProgram->GetUniformLoc(c.uniformStereoMatrix4.name);
-				}
-				if (loc >= 0) {
-					if (GetVRFBOIndex() == 0) {
-						glUniformMatrix4fv(loc, 1, false, c.uniformStereoMatrix4.mData);
-					} else {
-						glUniformMatrix4fv(loc, 1, false, c.uniformStereoMatrix4.mData + 16);
-					}
-				}
-				if (GetVRFBOIndex() == 1 || GetVRPassesCount() == 1) {
-					// Only delete the data if we're rendering the only or the second eye.
-					// If we delete during the first eye, we get a use-after-free or double delete.
-					delete[] c.uniformStereoMatrix4.mData;
+			int loc = c.uniformStereoMatrix4.loc ? *c.uniformStereoMatrix4.loc : -1;
+			if (c.uniformStereoMatrix4.name) {
+				loc = curProgram->GetUniformLoc(c.uniformStereoMatrix4.name);
+			}
+			if (loc >= 0) {
+				if (GetVRFBOIndex() == 0) {
+					glUniformMatrix4fv(loc, 1, false, c.uniformStereoMatrix4.mData);
+				} else {
+					glUniformMatrix4fv(loc, 1, false, c.uniformStereoMatrix4.mData + 16);
 				}
 			}
+			if (GetVRFBOIndex() == 1 || GetVRPassesCount() == 1) {
+				// Only delete the data if we're rendering the only or the second eye.
+				// If we delete during the first eye, we get a use-after-free or double delete.
+				delete[] c.uniformStereoMatrix4.mData;
+			}
 			CHECK_GL_ERROR_IF_DEBUG();
 			break;
 		}
@@ -1739,13 +1693,13 @@ void GLQueueRunner::fbo_ext_create(const GLRInitStep &step) {
 	GLenum status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 	switch (status) {
 	case GL_FRAMEBUFFER_COMPLETE_EXT:
-		// INFO_LOG(G3D, "Framebuffer verified complete.");
+		// INFO_LOG(Log::G3D, "Framebuffer verified complete.");
 		break;
 	case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
-		ERROR_LOG(G3D, "GL_FRAMEBUFFER_UNSUPPORTED");
+		ERROR_LOG(Log::G3D, "GL_FRAMEBUFFER_UNSUPPORTED");
 		break;
 	case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:
-		ERROR_LOG(G3D, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT ");
+		ERROR_LOG(Log::G3D, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT ");
 		break;
 	default:
 		_assert_msg_(false, "Other framebuffer error: %d", status);
