@@ -12,7 +12,7 @@
 #include "Common/Math/math_util.h"
 
 #if 0 // def _DEBUG
-#define VLOG(...) INFO_LOG(G3D, __VA_ARGS__)
+#define VLOG(...) INFO_LOG(Log::G3D, __VA_ARGS__)
 #else
 #define VLOG(...)
 #endif
@@ -59,7 +59,7 @@ void GLRenderManager::ThreadStart(Draw::DrawContext *draw) {
 	renderThreadId = std::this_thread::get_id();
 
 	if (newInflightFrames_ != -1) {
-		INFO_LOG(G3D, "Updating inflight frames to %d", newInflightFrames_);
+		INFO_LOG(Log::G3D, "Updating inflight frames to %d", newInflightFrames_);
 		inflightFrames_ = newInflightFrames_;
 		newInflightFrames_ = -1;
 	}
@@ -99,9 +99,12 @@ void GLRenderManager::ThreadStart(Draw::DrawContext *draw) {
 }
 
 void GLRenderManager::ThreadEnd() {
-	INFO_LOG(G3D, "ThreadEnd");
+	INFO_LOG(Log::G3D, "GLRenderManager::ThreadEnd begin");
+
+	runCompileThread_ = false;
 
 	queueRunner_.DestroyDeviceObjects();
+
 	VLOG("  PULL: Quitting");
 
 	// Good time to run all the deleters to get rid of leftover objects.
@@ -116,6 +119,7 @@ void GLRenderManager::ThreadEnd() {
 	}
 	steps_.clear();
 	initSteps_.clear();
+	INFO_LOG(Log::G3D, "GLRenderManager::ThreadEnd end");
 }
 
 // Unlike in Vulkan, this isn't a full independent function, instead it gets called every frame.
@@ -124,36 +128,31 @@ void GLRenderManager::ThreadEnd() {
 // at which point we can leave.
 //
 // NOTE: If run_ is true, we WILL run a task!
-bool GLRenderManager::ThreadFrame() {
-	if (!runCompileThread_) {
-		return false;
-	}
+bool GLRenderManager::ThreadFrame(bool waitIfEmpty) {
+	_assert_(runCompileThread_);
 
 	GLRRenderThreadTask *task = nullptr;
 
 	// In case of syncs or other partial completion, we keep going until we complete a frame.
 	while (true) {
-		// Pop a task of the queue and execute it.
-		// NOTE: We need to actually wait for a task, we can't just bail!
+		// Pop a task off the queue and execute it. Exiting this loop is done with a special EXIT task,
+		// to keep things uniform.
 		{
 			std::unique_lock<std::mutex> lock(pushMutex_);
-			while (renderThreadQueue_.empty()) {
-				pushCondVar_.wait(lock);
-			}
-			task = std::move(renderThreadQueue_.front());
-			renderThreadQueue_.pop();
-		}
 
-		// We got a task! We can now have pushMutex_ unlocked, allowing the host to
-		// push more work when it feels like it, and just start working.
-		if (task->runType == GLRRunType::EXIT) {
-			delete task;
-			// Oh, host wanted out. Let's leave, and also let's notify the host.
-			// This is unlike Vulkan too which can just block on the thread existing.
-			std::unique_lock<std::mutex> lock(syncMutex_);
-			syncCondVar_.notify_one();
-			syncDone_ = true;
-			break;
+			if (!waitIfEmpty && renderThreadQueue_.empty()) {
+				lock.unlock();
+				// Oh, host wanted out. Let's leave, and also let's notify the host.
+				// This is unlike Vulkan too which can just block on the thread existing.
+				std::unique_lock<std::mutex> lock(syncMutex_);
+				syncCondVar_.notify_one();
+				syncDone_ = true;
+				return false;
+			}
+
+			pushCondVar_.wait(lock, [this] { return !renderThreadQueue_.empty(); });
+			task = renderThreadQueue_.front();
+			renderThreadQueue_.pop();
 		}
 
 		// Render the scene.
@@ -169,17 +168,13 @@ bool GLRenderManager::ThreadFrame() {
 	return true;
 }
 
-void GLRenderManager::StopThread() {
+void GLRenderManager::StartThread() {
 	// There's not really a lot to do here anymore.
-	INFO_LOG(G3D, "GLRenderManager::StopThread()");
-	if (runCompileThread_) {
-		runCompileThread_ = false;
-
-		std::unique_lock<std::mutex> lock(pushMutex_);
-		renderThreadQueue_.push(new GLRRenderThreadTask(GLRRunType::EXIT));
-		pushCondVar_.notify_one();
+	INFO_LOG(Log::G3D, "GLRenderManager::StartThread()");
+	if (!runCompileThread_) {
+		runCompileThread_ = true;
 	} else {
-		WARN_LOG(G3D, "GL submission thread was already paused.");
+		INFO_LOG(Log::G3D, "GL submission thread was already running.");
 	}
 }
 
@@ -254,6 +249,7 @@ void GLRenderManager::BindFramebufferAsRenderTarget(GLRFramebuffer *fb, GLRRende
 	}
 }
 
+// aspectBit: GL_COLOR_BUFFER_BIT etc
 void GLRenderManager::BindFramebufferAsTexture(GLRFramebuffer *fb, int binding, int aspectBit) {
 	_dbg_assert_(curRenderStep_ && curRenderStep_->stepType == GLRStepType::RENDER);
 	_dbg_assert_(binding < MAX_GL_TEXTURE_SLOTS);
@@ -348,6 +344,9 @@ void GLRenderManager::BeginFrame(bool enableProfiling) {
 	curProgram_ = nullptr;
 #endif
 
+	// Shouldn't call BeginFrame unless we're in a run state.
+	_dbg_assert_(runCompileThread_);
+
 	int curFrame = GetCurFrame();
 
 	FrameTimeData &frameTimeData = frameTimeHistory_.Add(frameIdGen_);
@@ -368,10 +367,6 @@ void GLRenderManager::BeginFrame(bool enableProfiling) {
 		frameData.readyForFence = false;
 	}
 
-	if (!runCompileThread_) {
-		WARN_LOG(G3D, "BeginFrame while !run_!");
-	}
-
 	insideFrame_ = true;
 }
 
@@ -407,9 +402,9 @@ void GLRenderManager::Finish() {
 	task->frame = curFrame;
 	{
 		std::unique_lock<std::mutex> lock(pushMutex_);
+		task->initSteps = std::move(initSteps_);
+		task->steps = std::move(steps_);
 		renderThreadQueue_.push(task);
-		renderThreadQueue_.back()->initSteps = std::move(initSteps_);
-		renderThreadQueue_.back()->steps = std::move(steps_);
 		initSteps_.clear();
 		steps_.clear();
 		pushCondVar_.notify_one();
