@@ -199,7 +199,6 @@ enum class GLRRunType {
 	SUBMIT,
 	PRESENT,
 	SYNC,
-	EXIT,
 };
 
 class GLRenderManager;
@@ -237,7 +236,7 @@ public:
 
 	void ThreadStart(Draw::DrawContext *draw);
 	void ThreadEnd();
-	bool ThreadFrame();  // Returns true if it did anything. False means the queue was empty.
+	bool ThreadFrame(bool waitIfEmpty);  // Returns true if it did anything. False means the queue was empty.
 
 	void SetErrorCallback(ErrorCallbackFn callback, void *userdata) {
 		queueRunner_.SetErrorCallback(callback, userdata);
@@ -825,7 +824,6 @@ public:
 	}
 
 	void StartThread();  // Currently only used on iOS, since we fully recreate the context on Android
-	void StopThread();
 
 	bool SawOutOfMemory() {
 		return queueRunner_.SawOutOfMemory();
@@ -884,8 +882,6 @@ private:
 	std::mutex syncMutex_;
 	std::condition_variable syncCondVar_;
 
-	bool firstFrame_ = true;
-	bool vrRenderStarted_ = false;
 	bool syncDone_ = false;
 
 	GLDeleter deleter_;
