@@ -25,10 +25,6 @@
 
 enum class VulkanInitFlags : uint32_t {
 	VALIDATE = (1 << 0),
-	PRESENT_MAILBOX = (1 << 1),
-	PRESENT_IMMEDIATE = (1 << 2),
-	PRESENT_FIFO_RELAXED = (1 << 3),
-	PRESENT_FIFO = (1 << 4),
 	DISABLE_IMPLICIT_LAYERS = (1 << 5),
 };
 ENUM_CLASS_BITOPS(VulkanInitFlags);
@@ -56,6 +52,7 @@ template<class R, class T> inline void ChainStruct(R &root, T *newStruct) {
 
 // Not all will be usable on all platforms, of course...
 enum WindowSystem {
+	WINDOWSYSTEM_UNINITIALIZED,
 #ifdef _WIN32
 	WINDOWSYSTEM_WIN32,
 #endif
@@ -184,6 +181,7 @@ public:
 		const char *app_name;
 		int app_ver;
 		VulkanInitFlags flags;
+		std::string customDriver;
 	};
 
 	VkResult CreateInstance(const CreateInfo &info);
@@ -207,8 +205,10 @@ public:
 
 	VkDevice GetDevice() const { return device_; }
 	VkInstance GetInstance() const { return instance_; }
-	VulkanInitFlags GetInitFlags() const { return flags_; }
-	void UpdateInitFlags(VulkanInitFlags flags) { flags_ = flags; }
+	VulkanInitFlags GetInitFlags() const { return createInfo_.flags; }
+
+	// Of course, this won't update things that can only change on first init.
+	void UpdateCreateInfo(const VulkanContext::CreateInfo &info) { createInfo_ = info; }
 
 	VulkanDeleteList &Delete() { return globalDeleteList_; }
 
@@ -217,7 +217,8 @@ public:
 	VkResult InitSurface(WindowSystem winsys, void *data1, void *data2);
 	VkResult ReinitSurface();
 
-	bool InitSwapchain();
+	// If the present mode is not available, will fall back to the first available (which is almost always FIFO).
+	bool InitSwapchain(VkPresentModeKHR desiredPresentMode);
 	void SetCbGetDrawSize(std::function<VkExtent2D()>);
 
 	void DestroySwapchain();
@@ -287,6 +288,7 @@ public:
 		VkPhysicalDevicePresentWaitFeaturesKHR presentWait;
 		VkPhysicalDevicePresentIdFeaturesKHR presentId;
 		VkPhysicalDeviceProvokingVertexFeaturesEXT provokingVertex;
+		VkPhysicalDevicePresentModeFifoLatestReadyFeaturesKHR presentModeFifoProps;
 	};
 
 	const PhysicalDeviceProps &GetPhysicalDeviceProperties(int i = -1) const {
@@ -414,6 +416,15 @@ public:
 		return availablePresentModes_;
 	}
 
+	bool PresentModeSupported(VkPresentModeKHR mode) const {
+		for (const auto &m : availablePresentModes_) {
+			if (m == mode) {
+				return true;
+			}
+		}
+		return false;
+	}
+
 	int GetLastDeleteCount() const {
 		return frame_[curFrame_].deleteList.GetLastDeleteCount();
 	}
@@ -441,7 +452,7 @@ private:
 
 	bool CheckLayers(const std::vector<LayerProperties> &layer_props, const std::vector<const char *> &layer_names) const;
 
-	WindowSystem winsys_{};
+	WindowSystem winsys_ = WINDOWSYSTEM_UNINITIALIZED;
 
 	// Don't use the real types here to avoid having to include platform-specific stuff
 	// that we really don't want in everything that uses VulkanContext.
@@ -486,7 +497,8 @@ private:
 	// Swap chain extent
 	VkExtent2D swapChainExtent_{};
 
-	VulkanInitFlags flags_{};
+	VulkanContext::CreateInfo createInfo_{};
+
 	PerfClass devicePerfClass_ = PerfClass::SLOW;
 
 	int inflightFrames_ = MAX_INFLIGHT_FRAMES;
