@@ -577,6 +577,7 @@ struct PipelineDesc {
 	const Slice<SamplerDef> samplers;
 };
 
+// Not normally used as a bitfield, but useful as a capability flag.
 enum class PresentMode {
 	FIFO = 1,
 	IMMEDIATE = 2,
@@ -584,10 +585,17 @@ enum class PresentMode {
 };
 ENUM_CLASS_BITOPS(PresentMode);
 
+inline bool PresentationModeBlocks(PresentMode mode) {
+	return mode & PresentMode::FIFO;
+}
+
 struct DeviceCaps {
 	GPUVendor vendor;
 	uint32_t deviceID;  // use caution!
 
+	uint32_t maxTextureSize;  // largest side.
+	uint32_t maxClipPlanes;
+
 	CoordConvention coordConvention;
 	DataFormat preferredDepthBufferFormat;
 	DataFormat preferredShadowMapFormatLow;
@@ -627,7 +635,7 @@ struct DeviceCaps {
 	// Old style, for older GL or Direct3D 9.
 	u32 clipPlanesSupported;
 
-	// Presentation caps
+	// Presentation caps (simplified). Note: Vulkan supports more, we handle that separately now.
 	int presentMaxInterval; // 1 on many backends
 	bool presentInstantModeChange;
 	PresentMode presentModesSupported;
@@ -644,6 +652,7 @@ enum class TextureSwizzle {
 	DEFAULT,
 	R8_AS_ALPHA,
 	R8_AS_GRAYSCALE,
+	R8_AS_PREMUL_ALPHA,
 };
 
 struct TextureDesc {
@@ -852,9 +861,9 @@ public:
 	virtual void BeginFrame(DebugFlags debugFlags) = 0;
 	virtual void EndFrame() = 0;
 
-	// vblanks is only relevant in FIFO present mode.
-	// NOTE: Not all backends support vblanks > 1. Some backends also can't change presentation mode immediately.
-	virtual void Present(PresentMode presentMode, int vblanks) = 0;
+	// Some backends also can't change presentation mode immediately.
+	virtual void Present(PresentMode presentMode) = 0;
+	virtual PresentMode GetCurrentPresentMode() const = 0;
 
 	// This should be avoided as much as possible, in favor of clearing when binding a render target, which is native
 	// on Vulkan.
