@@ -1,31 +1,14 @@
+#include <cmath>
+#include <cstdio>
+#include <cstdlib>
+
 #include "Common/Net/HTTPClient.h"
 
 #include "Common/TimeUtil.h"
 #include "Common/StringUtils.h"
 #include "Common/System/OSD.h"
 
-#ifndef _WIN32
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <sys/select.h>
-#include <sys/socket.h>
-#include <sys/types.h>
-#include <netdb.h>
-#include <unistd.h>
-#define closesocket close
-#else
-#ifndef NOMINMAX
-#define NOMINMAX
-#endif
-#include <winsock2.h>
-#include <ws2tcpip.h>
-#include <io.h>
-#endif
-
-#include <cmath>
-#include <cstdio>
-#include <cstdlib>
-
+#include "Common/Net/SocketCompat.h"
 #include "Common/Net/Resolve.h"
 #include "Common/Net/URL.h"
 
@@ -38,9 +21,6 @@
 
 namespace net {
 
-Connection::Connection() {
-}
-
 Connection::~Connection() {
 	Disconnect();
 	if (resolved_ != nullptr)
@@ -67,13 +47,17 @@ const char *DNSTypeAsString(DNSType type) {
 	}
 }
 
+std::string Connection::GetLocalIpAsString() const {
+	return fd_util::GetLocalIP(this->sock());
+}
+
 bool Connection::Resolve(const char *host, int port, DNSType type) {
 	if ((intptr_t)sock_ != -1) {
-		ERROR_LOG(IO, "Resolve: Already have a socket");
+		ERROR_LOG(Log::IO, "Resolve: Already have a socket");
 		return false;
 	}
 	if (!host || port < 1 || port > 65535) {
-		ERROR_LOG(IO, "Resolve: Invalid host or port (%d)", port);
+		ERROR_LOG(Log::IO, "Resolve: Invalid host or port (%d)", port);
 		return false;
 	}
 
@@ -83,9 +67,15 @@ bool Connection::Resolve(const char *host, int port, DNSType type) {
 	char port_str[16];
 	snprintf(port_str, sizeof(port_str), "%d", port);
 
+	std::string processedHostname(host);
+
+	if (customResolve_) {
+		processedHostname = customResolve_(host);
+	}
+	
 	std::string err;
-	if (!net::DNSResolve(host, port_str, &resolved_, err, type)) {
-		WARN_LOG(IO, "Failed to resolve host '%s': '%s' (%s)", host, err.c_str(), DNSTypeAsString(type));
+	if (!net::DNSResolve(processedHostname.c_str(), port_str, &resolved_, err, type)) {
+		WARN_LOG(Log::IO, "Failed to resolve host '%s': '%s' (%s)", host, err.c_str(), DNSTypeAsString(type));
 		// Zero port so that future calls fail.
 		port_ = 0;
 		return false;
@@ -108,7 +98,7 @@ static void FormatAddr(char *addrbuf, size_t bufsize, const addrinfo *info) {
 
 bool Connection::Connect(int maxTries, double timeout, bool *cancelConnect) {
 	if (port_ <= 0) {
-		ERROR_LOG(IO, "Bad port");
+		ERROR_LOG(Log::IO, "Bad port");
 		return false;
 	}
 	sock_ = -1;
@@ -124,13 +114,13 @@ bool Connection::Connect(int maxTries, double timeout, bool *cancelConnect) {
 
 			int sock = socket(possible->ai_family, SOCK_STREAM, IPPROTO_TCP);
 			if ((intptr_t)sock == -1) {
-				ERROR_LOG(IO, "Bad socket");
+				ERROR_LOG(Log::IO, "Bad socket");
 				continue;
 			}
 			// Windows sockets aren't limited by socket number, just by count, so checking FD_SETSIZE there is wrong.
 #if !PPSSPP_PLATFORM(WINDOWS)
 			if (sock >= FD_SETSIZE) {
-				ERROR_LOG(IO, "Socket doesn't fit in FD_SET: %d   We probably have a leak.", sock);
+				ERROR_LOG(Log::IO, "Socket doesn't fit in FD_SET: %d   We probably have a leak.", sock);
 				closesocket(sock);
 				continue;
 			}
@@ -140,24 +130,17 @@ bool Connection::Connect(int maxTries, double timeout, bool *cancelConnect) {
 			// Start trying to connect (async with timeout.)
 			errno = 0;
 			if (connect(sock, possible->ai_addr, (int)possible->ai_addrlen) < 0) {
-#if PPSSPP_PLATFORM(WINDOWS)
-				int errorCode = WSAGetLastError();
+				int errorCode = socket_errno;
 				std::string errorString = GetStringErrorMsg(errorCode);
-				bool unreachable = errorCode == WSAENETUNREACH;
-				bool inProgress = errorCode == WSAEINPROGRESS || errorCode == WSAEWOULDBLOCK;
-#else
-				int errorCode = errno;
-				std::string errorString = strerror(errno);
 				bool unreachable = errorCode == ENETUNREACH;
 				bool inProgress = errorCode == EINPROGRESS || errorCode == EWOULDBLOCK;
-#endif
 				if (!inProgress) {
 					char addrStr[128]{};
 					FormatAddr(addrStr, sizeof(addrStr), possible);
 					if (!unreachable) {
-						ERROR_LOG(HTTP, "connect(%d) call to %s failed (%d: %s)", sock, addrStr, errorCode, errorString.c_str());
+						ERROR_LOG(Log::HTTP, "connect(%d) call to %s failed (%d: %s)", sock, addrStr, errorCode, errorString.c_str());
 					} else {
-						INFO_LOG(HTTP, "connect(%d): Ignoring unreachable resolved address %s", sock, addrStr);
+						INFO_LOG(Log::HTTP, "connect(%d): Ignoring unreachable resolved address %s", sock, addrStr);
 					}
 					closesocket(sock);
 					continue;
@@ -173,7 +156,7 @@ bool Connection::Connect(int maxTries, double timeout, bool *cancelConnect) {
 		int selectResult = 0;
 		long timeoutHalfSeconds = floor(2 * timeout);
 		while (timeoutHalfSeconds >= 0 && selectResult == 0) {
-			struct timeval tv;
+			struct timeval tv{};
 			tv.tv_sec = 0;
 			if (timeoutHalfSeconds > 0) {
 				// Wait up to 0.5 seconds between cancel checks.
@@ -186,7 +169,7 @@ bool Connection::Connect(int maxTries, double timeout, bool *cancelConnect) {
 
 			selectResult = select(maxfd, nullptr, &fds, nullptr, &tv);
 			if (cancelConnect && *cancelConnect) {
-				WARN_LOG(HTTP, "connect: cancelled (1)");
+				WARN_LOG(Log::HTTP, "connect: cancelled (1): %s:%d", host_.c_str(), port_);
 				break;
 			}
 		}
@@ -210,11 +193,11 @@ bool Connection::Connect(int maxTries, double timeout, bool *cancelConnect) {
 		}
 
 		if (cancelConnect && *cancelConnect) {
-			WARN_LOG(HTTP, "connect: cancelled (2)");
+			WARN_LOG(Log::HTTP, "connect: cancelled (2): %s:%d", host_.c_str(), port_);
 			break;
 		}
 
-		sleep_ms(1);
+		sleep_ms(1, "connect");
 	}
 
 	// Nothing connected, unfortunately.
@@ -236,8 +219,9 @@ namespace http {
 constexpr const char *DEFAULT_USERAGENT = "PPSSPP";
 constexpr const char *HTTP_VERSION = "1.1";
 
-Client::Client() {
+Client::Client(net::ResolveFunc func) : Connection(func) {
 	userAgent_ = DEFAULT_USERAGENT;
+	httpVersion_ = HTTP_VERSION;
 }
 
 Client::~Client() {
@@ -246,8 +230,10 @@ Client::~Client() {
 
 // Ignores line folding (deprecated), but respects field combining.
 // Don't use for Set-Cookie, which is a special header per RFC 7230.
-bool GetHeaderValue(const std::vector<std::string> &responseHeaders, const std::string &header, std::string *value) {
-	std::string search = header + ":";
+bool GetHeaderValue(const std::vector<std::string> &responseHeaders, std::string_view header, std::string *value) {
+	std::string search(header);
+	search.push_back(':');
+
 	bool found = false;
 
 	value->clear();
@@ -262,7 +248,7 @@ bool GetHeaderValue(const std::vector<std::string> &responseHeaders, const std::
 			if (!found)
 				*value = stripped.substr(value_pos);
 			else
-				*value += "," + stripped.substr(value_pos);
+				*value += "," + std::string(stripped.substr(value_pos));
 			found = true;
 		}
 	}
@@ -270,15 +256,18 @@ bool GetHeaderValue(const std::vector<std::string> &responseHeaders, const std::
 	return found;
 }
 
-void DeChunk(Buffer *inbuffer, Buffer *outbuffer, int contentLength) {
+static bool DeChunk(Buffer *inbuffer, Buffer *outbuffer, int contentLength) {
+	_dbg_assert_(outbuffer->empty());
 	int dechunkedBytes = 0;
 	while (true) {
 		std::string line;
 		inbuffer->TakeLineCRLF(&line);
 		if (!line.size())
-			return;
+			return false;
 		unsigned int chunkSize = 0;
-		sscanf(line.c_str(), "%x", &chunkSize);
+		if (sscanf(line.c_str(), "%x", &chunkSize) != 1) {
+			return false;
+		}
 		if (chunkSize) {
 			std::string data;
 			inbuffer->Take(chunkSize, &data);
@@ -286,11 +275,13 @@ void DeChunk(Buffer *inbuffer, Buffer *outbuffer, int contentLength) {
 		} else {
 			// a zero size chunk should mean the end.
 			inbuffer->clear();
-			return;
+			return true;
 		}
 		dechunkedBytes += chunkSize;
 		inbuffer->Skip(2);
 	}
+	// Unreachable
+	return true;
 }
 
 int Client::GET(const RequestParams &req, Buffer *output, std::vector<std::string> &responseHeaders, net::RequestProgress *progress) {
@@ -320,13 +311,14 @@ int Client::GET(const RequestParams &req, Buffer *output, net::RequestProgress *
 	return code;
 }
 
-int Client::POST(const RequestParams &req, const std::string &data, const std::string &mime, Buffer *output, net::RequestProgress *progress) {
+int Client::POST(const RequestParams &req, std::string_view data, std::string_view mime, Buffer *output, net::RequestProgress *progress) {
 	char otherHeaders[2048];
 	if (mime.empty()) {
 		snprintf(otherHeaders, sizeof(otherHeaders), "Content-Length: %lld\r\n", (long long)data.size());
 	} else {
-		snprintf(otherHeaders, sizeof(otherHeaders), "Content-Length: %lld\r\nContent-Type: %s\r\n", (long long)data.size(), mime.c_str());
+		snprintf(otherHeaders, sizeof(otherHeaders), "Content-Length: %lld\r\nContent-Type: %.*s\r\n", (long long)data.size(), (int)mime.size(), mime.data());
 	}
+
 	int err = SendRequestWithData("POST", req, data, otherHeaders, progress);
 	if (err < 0) {
 		return err;
@@ -346,7 +338,7 @@ int Client::POST(const RequestParams &req, const std::string &data, const std::s
 	return code;
 }
 
-int Client::POST(const RequestParams &req, const std::string &data, Buffer *output, net::RequestProgress *progress) {
+int Client::POST(const RequestParams &req, std::string_view data, Buffer *output, net::RequestProgress *progress) {
 	return POST(req, data, "", output, progress);
 }
 
@@ -354,7 +346,7 @@ int Client::SendRequest(const char *method, const RequestParams &req, const char
 	return SendRequestWithData(method, req, "", otherHeaders, progress);
 }
 
-int Client::SendRequestWithData(const char *method, const RequestParams &req, const std::string &data, const char *otherHeaders, net::RequestProgress *progress) {
+int Client::SendRequestWithData(const char *method, const RequestParams &req, std::string_view data, const char *otherHeaders, net::RequestProgress *progress) {
 	progress->Update(0, 0, false);
 
 	net::Buffer buffer;
@@ -381,7 +373,7 @@ int Client::SendRequestWithData(const char *method, const RequestParams &req, co
 	return 0;
 }
 
-int Client::ReadResponseHeaders(net::Buffer *readbuf, std::vector<std::string> &responseHeaders, net::RequestProgress *progress) {
+int Client::ReadResponseHeaders(net::Buffer *readbuf, std::vector<std::string> &responseHeaders, net::RequestProgress *progress, std::string *statusLine) {
 	// Snarf all the data we can into RAM. A little unsafe but hey.
 	static constexpr float CANCEL_INTERVAL = 0.25f;
 	bool ready = false;
@@ -391,13 +383,18 @@ int Client::ReadResponseHeaders(net::Buffer *readbuf, std::vector<std::string> &
 			return -1;
 		ready = fd_util::WaitUntilReady(sock(), CANCEL_INTERVAL, false);
 		if (!ready && time_now_d() > endTimeout) {
-			ERROR_LOG(HTTP, "HTTP headers timed out");
+			ERROR_LOG(Log::HTTP, "HTTP headers timed out");
 			return -1;
 		}
 	};
 	// Let's hope all the headers are available in a single packet...
 	if (readbuf->Read(sock(), 4096) < 0) {
-		ERROR_LOG(HTTP, "Failed to read HTTP headers :(");
+		ERROR_LOG(Log::HTTP, "Failed to read HTTP headers :(");
+		return -1;
+	}
+
+	if (readbuf->empty()) {
+		ERROR_LOG(Log::HTTP, "Empty HTTP header read buffer :(");
 		return -1;
 	}
 
@@ -415,19 +412,23 @@ int Client::ReadResponseHeaders(net::Buffer *readbuf, std::vector<std::string> &
 	if (code_pos != line.npos) {
 		code = atoi(&line[code_pos]);
 	} else {
-		ERROR_LOG(HTTP, "Could not parse HTTP status code: %s", line.c_str());
+		ERROR_LOG(Log::HTTP, "Could not parse HTTP status code: '%s'", line.c_str());
 		return -1;
 	}
 
+	if (statusLine)
+		*statusLine = line;
+
 	while (true) {
 		int sz = readbuf->TakeLineCRLF(&line);
 		if (!sz || sz < 0)
 			break;
+		VERBOSE_LOG(Log::HTTP, "Header line: %s", line.c_str());
 		responseHeaders.push_back(line);
 	}
 
 	if (responseHeaders.size() == 0) {
-		ERROR_LOG(HTTP, "No HTTP response headers");
+		ERROR_LOG(Log::HTTP, "No HTTP response headers");
 		return -1;
 	}
 
@@ -464,7 +465,7 @@ int Client::ReadResponseEntity(net::Buffer *readbuf, const std::vector<std::stri
 	}
 
 	if (contentLength < 0) {
-		WARN_LOG(HTTP, "Negative content length %d", contentLength);
+		WARN_LOG(Log::HTTP, "Negative content length %d", contentLength);
 		// Just sanity checking...
 		contentLength = 0;
 	}
@@ -475,7 +476,11 @@ int Client::ReadResponseEntity(net::Buffer *readbuf, const std::vector<std::stri
 	// output now contains the rest of the reply. Dechunk it.
 	if (!output->IsVoid()) {
 		if (chunked) {
-			DeChunk(readbuf, output, contentLength);
+			if (!DeChunk(readbuf, output, contentLength)) {
+				ERROR_LOG(Log::HTTP, "Bad chunked data, couldn't read chunk size");
+				progress->Update(0, 0, true);
+				return -1;
+			}
 		} else {
 			output->Append(*readbuf);
 		}
@@ -486,7 +491,7 @@ int Client::ReadResponseEntity(net::Buffer *readbuf, const std::vector<std::stri
 			output->TakeAll(&compressed);
 			bool result = decompress_string(compressed, &decompressed);
 			if (!result) {
-				ERROR_LOG(HTTP, "Error decompressing using zlib");
+				ERROR_LOG(Log::HTTP, "Error decompressing using zlib");
 				progress->Update(0, 0, true);
 				return -1;
 			}
@@ -498,26 +503,29 @@ int Client::ReadResponseEntity(net::Buffer *readbuf, const std::vector<std::stri
 	return 0;
 }
 
-HTTPRequest::HTTPRequest(RequestMethod method, const std::string &url, const std::string &postData, const std::string &postMime, const Path &outfile, ProgressBarMode progressBarMode, std::string_view name)
-	: Request(method, url, name, &cancelled_, progressBarMode), postData_(postData), postMime_(postMime), outfile_(outfile) {
+HTTPRequest::HTTPRequest(RequestMethod method, std::string_view url, std::string_view postData, std::string_view postMime, const Path &outfile, RequestFlags flags, net::ResolveFunc customResolve, std::string_view name)
+	: Request(method, url, name, &cancelled_, flags), postData_(postData), postMime_(postMime), customResolve_(customResolve) {
+	outfile_ = outfile;
 }
 
 HTTPRequest::~HTTPRequest() {
-	g_OSD.RemoveProgressBar(url_, Failed() ? false : true, 0.5f);
+	g_OSD.RemoveProgressBar(url_, !failed_, 0.5f);
 
-	_assert_msg_(joined_, "Download destructed without join");
+	if (thread_.joinable()) {
+		_dbg_assert_msg_(false, "Download destructed without join");
+	}
 }
 
 void HTTPRequest::Start() {
-	thread_ = std::thread(std::bind(&HTTPRequest::Do, this));
+	thread_ = std::thread([this] { Do(); });
 }
 
 void HTTPRequest::Join() {
-	if (joined_) {
-		ERROR_LOG(HTTP, "Already joined thread!");
+	if (!thread_.joinable()) {
+		ERROR_LOG(Log::HTTP, "Already joined thread!");
+		_dbg_assert_(false);
 	}
 	thread_.join();
-	joined_ = true;
 }
 
 void HTTPRequest::SetFailed(int code) {
@@ -532,13 +540,13 @@ int HTTPRequest::Perform(const std::string &url) {
 		return -1;
 	}
 
-	http::Client client;
+	http::Client client(customResolve_);
 	if (!userAgent_.empty()) {
 		client.SetUserAgent(userAgent_);
 	}
 
 	if (!client.Resolve(fileUrl.Host().c_str(), fileUrl.Port())) {
-		ERROR_LOG(HTTP, "Failed resolving %s", url.c_str());
+		ERROR_LOG(Log::HTTP, "Failed resolving %s", url.c_str());
 		return -1;
 	}
 
@@ -547,7 +555,7 @@ int HTTPRequest::Perform(const std::string &url) {
 	}
 
 	if (!client.Connect(2, 20.0, &cancelled_)) {
-		ERROR_LOG(HTTP, "Failed connecting to server or cancelled.");
+		ERROR_LOG(Log::HTTP, "Failed connecting to server or cancelled (=%d).", cancelled_);
 		return -1;
 	}
 
@@ -563,14 +571,13 @@ int HTTPRequest::Perform(const std::string &url) {
 	}
 }
 
-std::string HTTPRequest::RedirectLocation(const std::string &baseUrl) {
+std::string HTTPRequest::RedirectLocation(const std::string &baseUrl) const {
 	std::string redirectUrl;
 	if (GetHeaderValue(responseHeaders_, "Location", &redirectUrl)) {
 		Url url(baseUrl);
 		url = url.Relative(redirectUrl);
 		redirectUrl = url.ToString();
 	}
-
 	return redirectUrl;
 }
 
@@ -582,6 +589,7 @@ void HTTPRequest::Do() {
 
 	std::string downloadURL = url_;
 	while (resultCode_ == 0) {
+		// This is where the new request is performed.
 		int resultCode = Perform(downloadURL);
 		if (resultCode == -1) {
 			SetFailed(resultCode);
@@ -591,7 +599,7 @@ void HTTPRequest::Do() {
 		if (resultCode == 301 || resultCode == 302 || resultCode == 303 || resultCode == 307 || resultCode == 308) {
 			std::string redirectURL = RedirectLocation(downloadURL);
 			if (redirectURL.empty()) {
-				ERROR_LOG(HTTP, "Could not find Location header for redirect");
+				ERROR_LOG(Log::HTTP, "Could not find Location header for redirect");
 				resultCode_ = resultCode;
 			} else if (redirectURL == downloadURL || redirectURL == url_) {
 				// Simple loop detected, bail out.
@@ -599,19 +607,23 @@ void HTTPRequest::Do() {
 			}
 
 			// Perform the next GET.
-			if (resultCode_ == 0)
-				INFO_LOG(HTTP, "Download of %s redirected to %s", downloadURL.c_str(), redirectURL.c_str());
+			if (resultCode_ == 0) {
+				INFO_LOG(Log::HTTP, "Download of %s redirected to %s", downloadURL.c_str(), redirectURL.c_str());
+				buffer_.clear();
+				responseHeaders_.clear();
+			}
 			downloadURL = redirectURL;
 			continue;
 		}
 
 		if (resultCode == 200) {
-			INFO_LOG(HTTP, "Completed requesting %s (storing result to %s)", url_.c_str(), outfile_.empty() ? "memory" : outfile_.c_str());
-			if (!outfile_.empty() && !buffer_.FlushToFile(outfile_)) {
-				ERROR_LOG(HTTP, "Failed writing download to '%s'", outfile_.c_str());
+			INFO_LOG(Log::HTTP, "Completed requesting %s (storing result to %s)", url_.c_str(), outfile_.empty() ? "memory" : outfile_.c_str());
+			bool clear = !(flags_ & RequestFlags::KeepInMemory);
+			if (!outfile_.empty() && !buffer_.FlushToFile(outfile_, clear)) {
+				ERROR_LOG(Log::HTTP, "Failed writing download to '%s'", outfile_.c_str());
 			}
 		} else {
-			ERROR_LOG(HTTP, "Error requesting '%s' (storing result to '%s'): %i", url_.c_str(), outfile_.empty() ? "memory" : outfile_.c_str(), resultCode);
+			ERROR_LOG(Log::HTTP, "Error requesting '%s' (storing result to '%s'): %i", url_.c_str(), outfile_.empty() ? "memory" : outfile_.c_str(), resultCode);
 		}
 		resultCode_ = resultCode;
 	}
@@ -621,4 +633,4 @@ void HTTPRequest::Do() {
 	completed_ = true;
 }
 
-}	// http
+}  // namespace http
