@@ -20,12 +20,21 @@
 #include <cstdint>
 #include <cmath>
 #include <string>
+#include <string_view>
 #ifndef _MSC_VER
 #include <strings.h>
 #endif
 #include "Common/Common.h"
 #include "Common/CommonFuncs.h"
 
+struct ConfigBlock {
+	virtual ~ConfigBlock() = default;
+	virtual bool CanResetToDefault() const { return false; }
+	// If a block returns false here (like Config itself does), resetting to default will happen by the old per-setting mechanism.
+	virtual bool ResetToDefault(std::string_view blockName) { return false; }
+	virtual size_t Size() const { return sizeof(ConfigBlock); }  // For sanity checks
+};
+
 constexpr int PSP_MODEL_FAT = 0;
 constexpr int PSP_MODEL_SLIM = 1;
 constexpr int PSP_DEFAULT_FIRMWARE = 660;
@@ -48,11 +57,11 @@ int MultiplierToVolume100(float multiplier);
 float UIScaleFactorToMultiplier(int factor);
 
 struct ConfigTouchPos {
-	float x;
-	float y;
-	float scale;
+	float x = -1.0f;
+	float y = -1.0f;
+	float scale = 1.0f;
 	// Note: Show is not used for all settings.
-	bool show;
+	bool show = true;
 };
 
 struct ConfigCustomButton {
@@ -75,8 +84,8 @@ enum {
 	ROTATION_LOCKED_HORIZONTAL = 1,
 	ROTATION_LOCKED_VERTICAL = 2,
 	ROTATION_LOCKED_HORIZONTAL180 = 3,
-	ROTATION_LOCKED_VERTICAL180 = 4,
-	ROTATION_AUTO_HORIZONTAL = 5,
+	ROTATION_LOCKED_VERTICAL180 = 4,  // Deprecated
+	ROTATION_AUTO_HORIZONTAL = 5,     // Deprecated
 };
 
 enum TextureFiltering {
@@ -117,6 +126,12 @@ enum class DepthRasterMode {
 	FORCE_ON = 3,
 };
 
+enum class AudioSyncMode {
+	GRANULAR = 0,
+	CLASSIC_PITCH = 1,
+};
+
+// TODO: We can make this more fine-grained.
 enum class RestoreSettingsBits : int {
 	SETTINGS = 1,
 	CONTROLS = 2,
