@@ -17,9 +17,9 @@
 
 #include <algorithm>
 #include <atomic>
-#include <condition_variable>
 #include <cstring>
 #include <mutex>
+#include <condition_variable>
 #include <thread>
 
 #include "Common/Log.h"
@@ -268,7 +268,7 @@ void MemSlabMap::Slab::DoState(PointerWrap &p) {
 	} else {
 		std::string stringTag;
 		Do(p, stringTag);
-		truncate_cpy(tag, stringTag.c_str());
+		truncate_cpy(tag, stringTag);
 	}
 }
 
@@ -280,7 +280,7 @@ void MemSlabMap::Clear() {
 			delete s;
 		s = next;
 	}
-	delete bulkStorage_;
+	delete [] bulkStorage_;
 	bulkStorage_ = nullptr;
 	first_ = nullptr;
 	lastFind_ = nullptr;
@@ -525,9 +525,9 @@ void NotifyMemInfoPC(MemBlockFlags flags, uint32_t start, uint32_t size, uint32_
 
 	if (!(flags & MemBlockFlags::SKIP_MEMCHECK)) {
 		if (flags & MemBlockFlags::WRITE) {
-			CBreakPoints::ExecMemCheck(start, true, size, pc, tagStr);
+			g_breakpoints.ExecMemCheck(start, true, size, pc, tagStr);
 		} else if (flags & MemBlockFlags::READ) {
-			CBreakPoints::ExecMemCheck(start, false, size, pc, tagStr);
+			g_breakpoints.ExecMemCheck(start, false, size, pc, tagStr);
 		}
 	}
 }
@@ -541,7 +541,7 @@ void NotifyMemInfoCopy(uint32_t destPtr, uint32_t srcPtr, uint32_t size, const c
 		return;
 
 	bool needsFlush = false;
-	if (CBreakPoints::HasMemChecks()) {
+	if (g_breakpoints.HasMemChecks()) {
 		// This will cause a flush, but it's needed to trigger memchecks with proper data.
 		char tagData[128];
 		size_t tagSize = FormatMemWriteTagAt(tagData, sizeof(tagData), prefix, srcPtr, size);
