@@ -302,6 +302,8 @@ enum PSPErrorCode : u32 {
 	SCE_ERROR_MODULE_ALREADY_LOADED                   = 0x80111102,
 	SCE_ERROR_MODULE_NOT_LOADED                       = 0x80111103,
 	SCE_ERROR_AV_MODULE_BAD_ID                        = 0x80110F01,
+	SCE_ERROR_AV_MODULE_ALREADY_LOADED                = 0x80110F02,
+	SCE_ERROR_AV_MODULE_NOT_LOADED                    = 0x80110F03,
 
 	SCE_ERROR_UTILITY_INVALID_STATUS                  = 0x80110001,
 	SCE_ERROR_UTILITY_INVALID_PARAM_SIZE              = 0x80110004,
@@ -515,6 +517,20 @@ enum PSPErrorCode : u32 {
 	SCE_MP3_ERROR_BAD_ADDR = 0x80671002,
 	SCE_MP3_ERROR_BAD_SIZE = 0x80671003,
 
+	// pspnet_inet
+	SCE_NET_INET_ERROR_ALREADY_INITIALIZED = 0x80410201,
+	// Are these used?
+	SCE_NET_INET_ERROR_SOCKET_BUSY = 0x80410202,
+	SCE_NET_INET_ERROR_CONFIG_INVALID_ARG = 0x80410203,
+	SCE_NET_INET_ERROR_GET_IFADDR = 0x80410204,
+	SCE_NET_INET_ERROR_SET_IFADDR = 0x80410205,
+	SCE_NET_INET_ERROR_DEL_IFADDR = 0x80410206,
+	SCE_NET_INET_ERROR_NO_DEFAULT_ROUTE = 0x80410207,
+	SCE_NET_INET_ERROR_GET_ROUTE = 0x80410208,
+	SCE_NET_INET_ERROR_SET_ROUTE = 0x80410209,
+	SCE_NET_INET_ERROR_FLUSH_ROUTE = 0x8041020a,
+	SCE_NET_INET_ERROR_INVALID_ARG = 0x8041020b,
+
 	// pspnet_resolver
 	SCE_NET_RESOLVER_ERROR_NOT_TERMINATED = 0x80410401,
 	SCE_NET_RESOLVER_ERROR_NO_DNS_SERVER = 0x80410402,
@@ -552,4 +568,84 @@ enum PSPErrorCode : u32 {
 	SCE_NET_APCTL_ERROR_INVALID_ID = 0x80410a09,
 	SCE_NET_APCTL_ERROR_WLAN_SUSPENDED = 0x80410a0a,
 	SCE_NET_APCTL_ERROR_TIMEOUT = 0x80410a0b,
+
+	// pspnet_adhoc_auth
+	SCE_NET_ADHOC_ERROR_AUTH_ALREADY_INITIALIZED		= 0x80410601,
+
+	// pspnet_adhoc
+	SCE_NET_ADHOC_ERROR_INVALID_SOCKET_ID = 0x80410701,
+	SCE_NET_ADHOC_ERROR_INVALID_ADDR = 0x80410702,
+	SCE_NET_ADHOC_ERROR_INVALID_PORT = 0x80410703,
+	SCE_NET_ADHOC_ERROR_INVALID_BUFLEN = 0x80410704,
+	SCE_NET_ADHOC_ERROR_INVALID_DATALEN = 0x80410705,
+	SCE_NET_ADHOC_ERROR_NOT_ENOUGH_SPACE = 0x80400706, // not a typo
+	SCE_NET_ADHOC_ERROR_SOCKET_DELETED = 0x80410707,
+	SCE_NET_ADHOC_ERROR_SOCKET_ALERTED = 0x80410708,
+	SCE_NET_ADHOC_ERROR_WOULD_BLOCK = 0x80410709, //ERROR_NET_ADHOC_NO_DATA_AVAILABLE
+	SCE_NET_ADHOC_ERROR_PORT_IN_USE = 0x8041070a,
+	SCE_NET_ADHOC_ERROR_NOT_CONNECTED = 0x8041070B,
+	SCE_NET_ADHOC_ERROR_DISCONNECTED = 0x8041070c,
+	SCE_NET_ADHOC_ERROR_NOT_OPENED = 0x8040070D, // not a typo
+	SCE_NET_ADHOC_ERROR_NOT_LISTENED = 0x8040070E, // not a typo
+	SCE_NET_ADHOC_ERROR_SOCKET_ID_NOT_AVAIL = 0x8041070F,
+	SCE_NET_ADHOC_ERROR_PORT_NOT_AVAIL = 0x80410710,
+	SCE_NET_ADHOC_ERROR_INVALID_ARG = 0x80410711,
+	SCE_NET_ADHOC_ERROR_NOT_INITIALIZED = 0x80410712,
+	SCE_NET_ADHOC_ERROR_ALREADY_INITIALIZED = 0x80410713,
+	SCE_NET_ADHOC_ERROR_BUSY = 0x80410714,
+	SCE_NET_ADHOC_ERROR_TIMEOUT = 0x80410715,
+	SCE_NET_ADHOC_ERROR_NO_ENTRY = 0x80410716,
+	SCE_NET_ADHOC_ERROR_EXCEPTION_EVENT = 0x80410717,
+	SCE_NET_ADHOC_ERROR_CONNECTION_REFUSED = 0x80410718,
+	SCE_NET_ADHOC_ERROR_THREAD_ABORTED = 0x80410719,
+	SCE_NET_ADHOC_ERROR_ALREADY_CREATED = 0x8041071A,
+	SCE_NET_ADHOC_ERROR_NOT_IN_GAMEMODE = 0x8041071B,
+	SCE_NET_ADHOC_ERROR_NOT_CREATED	= 0x8041071C,
+
+	// pspnet_adhoc_matching
+	SCE_NET_ADHOC_MATCHING_ERROR_INVALID_MODE = 0x80410801,
+	SCE_NET_ADHOC_MATCHING_ERROR_INVALID_PORT = 0x80410802,
+	SCE_NET_ADHOC_MATCHING_ERROR_INVALID_MAXNUM = 0x80410803,
+	SCE_NET_ADHOC_MATCHING_ERROR_RXBUF_TOO_SHORT = 0x80410804,
+	SCE_NET_ADHOC_MATCHING_ERROR_INVALID_OPTLEN = 0x80410805,
+	SCE_NET_ADHOC_MATCHING_ERROR_INVALID_ARG = 0x80410806,
+	SCE_NET_ADHOC_MATCHING_ERROR_INVALID_ID = 0x80410807,
+	SCE_NET_ADHOC_MATCHING_ERROR_ID_NOT_AVAIL = 0x80410808,
+	SCE_NET_ADHOC_MATCHING_ERROR_NO_SPACE = 0x80410809,
+	SCE_NET_ADHOC_MATCHING_ERROR_IS_RUNNING = 0x8041080A,
+	SCE_NET_ADHOC_MATCHING_ERROR_NOT_RUNNING = 0x8041080B,
+	SCE_NET_ADHOC_MATCHING_ERROR_UNKNOWN_TARGET = 0x8041080C,
+	SCE_NET_ADHOC_MATCHING_ERROR_TARGET_NOT_READY = 0x8041080D,
+	SCE_NET_ADHOC_MATCHING_ERROR_EXCEED_MAXNUM = 0x8041080E,
+	SCE_NET_ADHOC_MATCHING_ERROR_REQUEST_IN_PROGRESS = 0x8041080F,
+	SCE_NET_ADHOC_MATCHING_ERROR_ALREADY_ESTABLISHED = 0x80410810,
+	SCE_NET_ADHOC_MATCHING_ERROR_BUSY = 0x80410811,
+	SCE_NET_ADHOC_MATCHING_ERROR_ALREADY_INITIALIZED = 0x80410812,
+	SCE_NET_ADHOC_MATCHING_ERROR_NOT_INITIALIZED = 0x80410813,
+	SCE_NET_ADHOC_MATCHING_ERROR_PORT_IN_USE = 0x80410814,
+	SCE_NET_ADHOC_MATCHING_ERROR_STACKSIZE_TOO_SHORT = 0x80410815,
+	SCE_NET_ADHOC_MATCHING_ERROR_INVALID_DATALEN = 0x80410816,
+	SCE_NET_ADHOC_MATCHING_ERROR_NOT_ESTABLISHED = 0x80410817,
+	SCE_NET_ADHOC_MATCHING_ERROR_DATA_BUSY = 0x80410818,
+
+	// pspnet_adhocctl
+	SCE_NET_ADHOCCTL_ERROR_NOT_LEFT_IBSS = 0x80410b01,
+	SCE_NET_ADHOCCTL_ERROR_ALREADY_CONNECTED = 0x80410b02,
+	SCE_NET_ADHOCCTL_ERROR_WLAN_SWITCH_OFF = 0x80410b03,
+	SCE_NET_ADHOCCTL_ERROR_INVALID_ARG = 0x80410B04,
+	SCE_NET_ADHOCCTL_ERROR_TIMEOUT = 0x80410b05,
+	SCE_NET_ADHOCCTL_ERROR_ID_NOT_FOUND = 0x80410B06,
+	SCE_NET_ADHOCCTL_ERROR_ALREADY_INITIALIZED = 0x80410b07,
+	SCE_NET_ADHOCCTL_ERROR_NOT_INITIALIZED = 0x80410b08,
+	SCE_NET_ADHOCCTL_ERROR_DISCONNECTED = 0x80410b09,
+	SCE_NET_ADHOCCTL_ERROR_NO_SCAN_INFO = 0x80410b0a,
+	SCE_NET_ADHOCCTL_ERROR_INVALID_IBSS = 0x80410b0b,
+	SCE_NET_ADHOCCTL_ERROR_NOT_ENTER_GAMEMODE = 0x80410B0C,
+	SCE_NET_ADHOCCTL_ERROR_CHANNEL_NOT_AVAILABLE = 0x80410B0D,
+	SCE_NET_ADHOCCTL_ERROR_WLAN_BEACON_LOST = 0x80410b0e,
+	SCE_NET_ADHOCCTL_ERROR_WLAN_SUSPENDED = 0x80410b0f,
+	SCE_NET_ADHOCCTL_ERROR_BUSY = 0x80410b10,
+	SCE_NET_ADHOCCTL_ERROR_CHANNEL_NOT_MATCH = 0x80410b11,
+	SCE_NET_ADHOCCTL_ERROR_TOO_MANY_HANDLERS = 0x80410b12,
+	SCE_NET_ADHOCCTL_ERROR_STACKSIZE_TOO_SHORT = 0x80410B13,
 };
