@@ -16,32 +16,19 @@
 // https://github.com/hrydgard/ppsspp and http://www.ppsspp.org/.
 
 #include "Common/Serialize/SerializeFuncs.h"
-#include "Common/Math/CrossSIMD.h"
-#include "Core/Config.h"
+#include "Common/Math/SIMDHeaders.h"
+#include "Common/StringUtils.h"
+#include "Core/System.h"
 #include "Core/Debugger/MemBlockInfo.h"
 #include "Core/HW/MediaEngine.h"
 #include "Core/MemMap.h"
-#include "Core/MIPS/MIPS.h"
 #include "Core/Reporting.h"
 #include "GPU/GPUState.h"  // Used by TextureDecoder.h when templates get instanced
 #include "GPU/Common/TextureDecoder.h"
-#include "GPU/GPUInterface.h"
 #include "Core/HW/SimpleAudioDec.h"
 
 #include <algorithm>
 
-#ifdef _M_SSE
-#include <emmintrin.h>
-#endif
-
-#if PPSSPP_ARCH(ARM_NEON)
-#if defined(_MSC_VER) && PPSSPP_ARCH(ARM64)
-#include <arm64_neon.h>
-#else
-#include <arm_neon.h>
-#endif
-#endif
-
 #ifdef USE_FFMPEG
 
 extern "C" {
@@ -51,6 +38,10 @@ extern "C" {
 #include "libavutil/imgutils.h"
 #include "libswscale/swscale.h"
 
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+	// private libavformat api (see demux.h in ffmpeg src tree)
+	void avpriv_stream_set_need_parsing(AVStream *st, enum AVStreamParseType type);
+#endif
 }
 #endif // USE_FFMPEG
 
@@ -71,7 +62,7 @@ static AVPixelFormat getSwsFormat(int pspFormat)
 	case GE_CMODE_32BIT_ABGR8888:
 		return AV_PIX_FMT_RGBA;
 	default:
-		ERROR_LOG(ME, "Unknown pixel format");
+		ERROR_LOG(Log::ME, "Unknown pixel format");
 		return (AVPixelFormat)0;
 	}
 }
@@ -96,11 +87,16 @@ void ffmpeg_logger(void *, int level, const char *format, va_list va_args) {
 
 	// Let's color the log line appropriately.
 	if (level <= AV_LOG_PANIC) {
-		ERROR_LOG(ME, "FF: %s", tmp);
+		ERROR_LOG(Log::ME, "FF: %s", tmp);
 	} else if (level >= AV_LOG_VERBOSE) {
-		DEBUG_LOG(ME, "FF: %s", tmp);
+		DEBUG_LOG(Log::ME, "FF: %s", tmp);
 	} else {
-		INFO_LOG(ME, "FF: %s", tmp);
+		// Downgrade some log messages we don't care about
+		if (startsWith(tmp, "No accelerated colorspace") || startsWith(tmp, "SEI type 1 size 40 truncated at 36")) {
+			VERBOSE_LOG(Log::ME, "FF: %s", tmp);
+		} else {
+			INFO_LOG(Log::ME, "FF: %s", tmp);
+		}
 	}
 }
 
@@ -128,7 +124,7 @@ static int getPixelFormatBytes(int pspFormat)
 		return 4;
 
 	default:
-		ERROR_LOG(ME, "Unknown pixel format");
+		ERROR_LOG(Log::ME, "Unknown pixel format");
 		return 4;
 	}
 }
@@ -146,10 +142,8 @@ MediaEngine::~MediaEngine() {
 
 void MediaEngine::closeMedia() {
 	closeContext();
-	if (m_pdata)
-		delete m_pdata;
-	if (m_demux)
-		delete m_demux;
+	delete m_pdata;
+	delete m_demux;
 	m_pdata = nullptr;
 	m_demux = nullptr;
 	AudioClose(&m_audioContext);
@@ -245,13 +239,13 @@ bool MediaEngine::SetupStreams() {
 #ifdef USE_FFMPEG
 	const u32 magic = *(u32_le *)&m_mpegheader[0];
 	if (magic != PSMF_MAGIC) {
-		WARN_LOG_REPORT(ME, "Could not setup streams, bad magic: %08x", magic);
+		WARN_LOG_REPORT(Log::ME, "Could not setup streams, bad magic: %08x", magic);
 		return false;
 	}
 	int numStreams = *(u16_be *)&m_mpegheader[0x80];
 	if (numStreams <= 0 || numStreams > 8) {
 		// Looks crazy.  Let's bail out and let FFmpeg handle it.
-		WARN_LOG_REPORT(ME, "Could not setup streams, unexpected stream count: %d", numStreams);
+		WARN_LOG_REPORT(Log::ME, "Could not setup streams, unexpected stream count: %d", numStreams);
 		return false;
 	}
 
@@ -303,18 +297,20 @@ bool MediaEngine::openContext(bool keepReadPos) {
 	}
 	av_dict_free(&open_opt);
 
-	if (!SetupStreams()) {
+	bool usedFFMPEGFindStreamInfo = false;
+	if (!SetupStreams() || PSP_CoreParameter().compat.flags().UseFFMPEGFindStreamInfo) {
 		// Fallback to old behavior.  Reads too much and corrupts when game doesn't read fast enough.
 		// SetupStreams sometimes work for newer FFmpeg 3.1+ now, but sometimes framerate is missing.
-		WARN_LOG_REPORT_ONCE(setupStreams, ME, "Failed to read valid video stream data from header");
+		WARN_LOG_REPORT_ONCE(setupStreams, Log::ME, "Failed to read valid video stream data from header");
 		if (avformat_find_stream_info(m_pFormatCtx, nullptr) < 0) {
 			closeContext();
 			return false;
 		}
+		usedFFMPEGFindStreamInfo = true;
 	}
 
 	if (m_videoStream >= (int)m_pFormatCtx->nb_streams) {
-		WARN_LOG_REPORT(ME, "Bad video stream %d", m_videoStream);
+		WARN_LOG_REPORT(Log::ME, "Bad video stream %d", m_videoStream);
 		m_videoStream = -1;
 	}
 
@@ -340,14 +336,18 @@ bool MediaEngine::openContext(bool keepReadPos) {
 		return false;
 
 	setVideoDim();
-	m_audioContext = new SimpleAudio(m_audioType, 44100, 2);
+	m_audioContext = CreateAudioDecoder((PSPAudioType)m_audioType);
 	m_isVideoEnd = false;
+
+	if (PSP_CoreParameter().compat.flags().UseFFMPEGFindStreamInfo && usedFFMPEGFindStreamInfo) {
+		m_mpegheaderReadPos++;
+		av_seek_frame(m_pFormatCtx, m_videoStream, 0, 0);
+	}
 #endif // USE_FFMPEG
 	return true;
 }
 
-void MediaEngine::closeContext()
-{
+void MediaEngine::closeContext() {
 #ifdef USE_FFMPEG
 	if (m_buffer)
 		av_free(m_buffer);
@@ -359,7 +359,7 @@ void MediaEngine::closeContext()
 		av_free(m_pIOContext->buffer);
 	if (m_pIOContext)
 		av_free(m_pIOContext);
-	for (auto it : m_pCodecCtxs) {
+	for (auto &it : m_pCodecCtxs) {
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 33, 100)
 		avcodec_free_context(&it.second);
 #else
@@ -368,8 +368,12 @@ void MediaEngine::closeContext()
 	}
 	m_pCodecCtxs.clear();
 	// These are streams allocated from avformat_new_stream.
-	for (auto it : m_codecsToClose) {
+	for (auto &it : m_codecsToClose) {
+#if LIBAVCODEC_VERSION_MAJOR >= 62
+		avcodec_free_context(&it);
+#else
 		avcodec_close(it);
+#endif
 	}
 	m_codecsToClose.clear();
 	if (m_pFormatCtx)
@@ -423,6 +427,9 @@ bool MediaEngine::addVideoStream(int streamNum, int streamId) {
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 33, 100)
 			stream->codecpar->codec_type = AVMEDIA_TYPE_VIDEO;
 			stream->codecpar->codec_id = AV_CODEC_ID_H264;
+#endif
+#if LIBAVFORMAT_VERSION_MAJOR >= 59
+			avpriv_stream_set_need_parsing(stream, AVSTREAM_PARSE_FULL);
 #else
 			stream->request_probe = 0;
 			stream->need_parsing = AVSTREAM_PARSE_FULL;
@@ -523,13 +530,13 @@ bool MediaEngine::setVideoStream(int streamNum, bool force) {
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 33, 100)
 		AVCodec *pCodec = avcodec_find_decoder(stream->codecpar->codec_id);
 		if (!pCodec) {
-			WARN_LOG_REPORT(ME, "Could not find decoder for %d", (int)stream->codecpar->codec_id);
+			WARN_LOG_REPORT(Log::ME, "Could not find decoder for %d", (int)stream->codecpar->codec_id);
 			return false;
 		}
 		AVCodecContext *m_pCodecCtx = avcodec_alloc_context3(pCodec);
 		int paramResult = avcodec_parameters_to_context(m_pCodecCtx, stream->codecpar);
 		if (paramResult < 0) {
-			WARN_LOG_REPORT(ME, "Failed to prepare context parameters: %08x", paramResult);
+			WARN_LOG_REPORT(Log::ME, "Failed to prepare context parameters: %08x", paramResult);
 			return false;
 		}
 #else
@@ -687,7 +694,7 @@ bool MediaEngine::stepVideo(int videoPixelMode, bool skipFrame) {
 				avcodec_send_packet(m_pCodecCtx, &packet);
 			int result = avcodec_receive_frame(m_pCodecCtx, m_pFrame);
 			if (result == 0) {
-				result = m_pFrame->pkt_size;
+				result = 1;
 				frameFinished = 1;
 			} else if (result == AVERROR(EAGAIN)) {
 				result = 0;
@@ -712,7 +719,10 @@ bool MediaEngine::stepVideo(int videoPixelMode, bool skipFrame) {
 						m_pCodecCtx->height, m_pFrameRGB->data, m_pFrameRGB->linesize);
 				}
 
-#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(55, 58, 100)
+#if LIBAVUTIL_VERSION_MAJOR >= 59
+				int64_t bestPts = m_pFrame->best_effort_timestamp;
+				int64_t ptsDuration = m_pFrame->duration;
+#elif LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(55, 58, 100)
 				int64_t bestPts = m_pFrame->best_effort_timestamp;
 				int64_t ptsDuration = m_pFrame->pkt_duration;
 #else
@@ -841,7 +851,7 @@ int MediaEngine::writeVideoImage(u32 bufferPtr, int frameWidth, int videoPixelMo
 
 	if (!Memory::IsValidRange(bufferPtr, videoImageSize) || frameWidth > 2048) {
 		// Clearly invalid values.  Let's just not.
-		ERROR_LOG_REPORT(ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
+		ERROR_LOG_REPORT(Log::ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
 		return 0;
 	}
 
@@ -892,7 +902,7 @@ int MediaEngine::writeVideoImage(u32 bufferPtr, int frameWidth, int videoPixelMo
 		break;
 
 	default:
-		ERROR_LOG_REPORT(ME, "Unsupported video pixel format %d", videoPixelMode);
+		ERROR_LOG_REPORT(Log::ME, "Unsupported video pixel format %d", videoPixelMode);
 		break;
 	}
 
@@ -930,7 +940,7 @@ int MediaEngine::writeVideoImageWithRange(u32 bufferPtr, int frameWidth, int vid
 
 	if (!Memory::IsValidRange(bufferPtr, videoImageSize) || frameWidth > 2048) {
 		// Clearly invalid values.  Let's just not.
-		ERROR_LOG_REPORT(ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
+		ERROR_LOG_REPORT(Log::ME, "Ignoring invalid video decode address %08x/%x", bufferPtr, frameWidth);
 		return 0;
 	}
 
@@ -992,12 +1002,12 @@ int MediaEngine::writeVideoImageWithRange(u32 bufferPtr, int frameWidth, int vid
 		break;
 
 	default:
-		ERROR_LOG_REPORT(ME, "Unsupported video pixel format %d", videoPixelMode);
+		ERROR_LOG_REPORT(Log::ME, "Unsupported video pixel format %d", videoPixelMode);
 		break;
 	}
 
 	if (swizzle) {
-		WARN_LOG_REPORT_ONCE(vidswizzle, ME, "Swizzling Video with range");
+		WARN_LOG_REPORT_ONCE(vidswizzle, Log::ME, "Swizzling Video with range");
 
 		const int bxc = videoLineSize / 16;
 		int byc = (height + 7) / 8;
@@ -1054,9 +1064,9 @@ int MediaEngine::getNextAudioFrame(u8 **buf, int *headerCode1, int *headerCode2)
 }
 
 int MediaEngine::getAudioSamples(u32 bufferPtr) {
-	u8 *buffer = Memory::GetPointerWriteRange(bufferPtr, 8192);
+	int16_t *buffer = (int16_t *)Memory::GetPointerWriteRange(bufferPtr, 8192);
 	if (buffer == nullptr) {
-		ERROR_LOG_REPORT(ME, "Ignoring bad audio decode address %08x during video playback", bufferPtr);
+		ERROR_LOG_REPORT(Log::ME, "Ignoring bad audio decode address %08x during video playback", bufferPtr);
 	}
 	if (!m_demux) {
 		return 0;
@@ -1068,7 +1078,7 @@ int MediaEngine::getAudioSamples(u32 bufferPtr) {
 	if (frameSize == 0) {
 		return 0;
 	}
-	int outbytes = 0;
+	int outSamples = 0;
 
 	if (m_audioContext != nullptr) {
 		if (headerCode1 == 0x24) {
@@ -1077,11 +1087,13 @@ int MediaEngine::getAudioSamples(u32 bufferPtr) {
 			m_audioContext->SetChannels(1);
 		}
 
-		if (!m_audioContext->Decode(audioFrame, frameSize, buffer, &outbytes)) {
-			ERROR_LOG(ME, "Audio (%s) decode failed during video playback", GetCodecName(m_audioType));
+		int inbytesConsumed = 0;
+		if (!m_audioContext->Decode(audioFrame, frameSize, &inbytesConsumed, 2, buffer, &outSamples)) {
+			ERROR_LOG(Log::ME, "Audio (%s) decode failed during video playback", GetCodecName(m_audioType));
 		}
+		int outBytes = outSamples * sizeof(int16_t) * 2;
 
-		NotifyMemInfo(MemBlockFlags::WRITE, bufferPtr, outbytes, "VideoDecodeAudio");
+		NotifyMemInfo(MemBlockFlags::WRITE, bufferPtr, outBytes, "VideoDecodeAudio");
 	}
 
 	return 0x2000;
