@@ -79,7 +79,7 @@ bool HasToken();
 /// Called when the system is being shut down. If Shutdown() returns false, the shutdown should be aborted if possible.
 bool Shutdown();
 
-void DownloadImageIfMissing(const std::string &cache_key, std::string &&url);
+void DownloadImageIfMissing(const std::string &cache_key, std::string_view url);
 
 /// Called once a frame at vsync time on the CPU thread, during gameplay.
 void FrameUpdate();
