@@ -21,22 +21,26 @@
 #include "Common/Data/Convert/ColorConv.h"
 #include "Common/Profiler/Profiler.h"
 #include "Common/LogReporting.h"
+#include "Common/Math/SIMDHeaders.h"
 #include "Common/Math/CrossSIMD.h"
 #include "Common/Math/lin/matrix4x4.h"
+#include "Common/TimeUtil.h"
+#include "Core/System.h"
 #include "Core/Config.h"
+#include "GPU/GPUCommon.h"
 #include "GPU/Common/DrawEngineCommon.h"
 #include "GPU/Common/SplineCommon.h"
+#include "GPU/Common/DepthRaster.h"
 #include "GPU/Common/VertexDecoderCommon.h"
+#include "GPU/Common/SoftwareTransformCommon.h"
 #include "GPU/ge_constants.h"
 #include "GPU/GPUState.h"
 
-#define QUAD_INDICES_MAX 65536
-
 enum {
-	TRANSFORMED_VERTEX_BUFFER_SIZE = VERTEX_BUFFER_MAX * sizeof(TransformedVertex)
+	TRANSFORMED_VERTEX_BUFFER_SIZE = VERTEX_BUFFER_MAX * sizeof(TransformedVertex),
 };
 
-DrawEngineCommon::DrawEngineCommon() : decoderMap_(16) {
+DrawEngineCommon::DrawEngineCommon() : decoderMap_(32) {
 	if (g_Config.bVertexDecoderJit && (g_Config.iCpuCore == (int)CPUCore::JIT || g_Config.iCpuCore == (int)CPUCore::JIT_IR)) {
 		decJitCache_ = new VertexDecoderJitCache();
 	}
@@ -44,6 +48,9 @@ DrawEngineCommon::DrawEngineCommon() : decoderMap_(16) {
 	transformedExpanded_ = (TransformedVertex *)AllocateMemoryPages(3 * TRANSFORMED_VERTEX_BUFFER_SIZE, MEM_PROT_READ | MEM_PROT_WRITE);
 	decoded_ = (u8 *)AllocateMemoryPages(DECODED_VERTEX_BUFFER_SIZE, MEM_PROT_READ | MEM_PROT_WRITE);
 	decIndex_ = (u16 *)AllocateMemoryPages(DECODED_INDEX_BUFFER_SIZE, MEM_PROT_READ | MEM_PROT_WRITE);
+	indexGen.Setup(decIndex_);
+
+	InitDepthRaster();
 }
 
 DrawEngineCommon::~DrawEngineCommon() {
@@ -51,6 +58,7 @@ DrawEngineCommon::~DrawEngineCommon() {
 	FreeMemoryPages(decIndex_, DECODED_INDEX_BUFFER_SIZE);
 	FreeMemoryPages(transformed_, TRANSFORMED_VERTEX_BUFFER_SIZE);
 	FreeMemoryPages(transformedExpanded_, 3 * TRANSFORMED_VERTEX_BUFFER_SIZE);
+	ShutdownDepthRaster();
 	delete decJitCache_;
 	decoderMap_.Iterate([&](const uint32_t vtype, VertexDecoder *decoder) {
 		delete decoder;
@@ -62,17 +70,6 @@ void DrawEngineCommon::Init() {
 	NotifyConfigChanged();
 }
 
-VertexDecoder *DrawEngineCommon::GetVertexDecoder(u32 vtype) {
-	VertexDecoder *dec;
-	if (decoderMap_.Get(vtype, &dec))
-		return dec;
-	dec = new VertexDecoder();
-	_assert_(dec);
-	dec->SetVertexType(vtype, decOptions_, decJitCache_);
-	decoderMap_.Insert(vtype, dec);
-	return dec;
-}
-
 std::vector<std::string> DrawEngineCommon::DebugGetVertexLoaderIDs() {
 	std::vector<std::string> ids;
 	decoderMap_.Iterate([&](const uint32_t vtype, VertexDecoder *decoder) {
@@ -95,30 +92,6 @@ std::string DrawEngineCommon::DebugGetVertexLoaderString(std::string id, DebugSh
 	}
 }
 
-static Vec3f ClipToScreen(const Vec4f& coords) {
-	float xScale = gstate.getViewportXScale();
-	float xCenter = gstate.getViewportXCenter();
-	float yScale = gstate.getViewportYScale();
-	float yCenter = gstate.getViewportYCenter();
-	float zScale = gstate.getViewportZScale();
-	float zCenter = gstate.getViewportZCenter();
-
-	float x = coords.x * xScale / coords.w + xCenter;
-	float y = coords.y * yScale / coords.w + yCenter;
-	float z = coords.z * zScale / coords.w + zCenter;
-
-	// 16 = 0xFFFF / 4095.9375
-	return Vec3f(x * 16 - gstate.getOffsetX16(), y * 16 - gstate.getOffsetY16(), z);
-}
-
-static Vec3f ScreenToDrawing(const Vec3f& coords) {
-	Vec3f ret;
-	ret.x = coords.x * (1.0f / 16.0f);
-	ret.y = coords.y * (1.0f / 16.0f);
-	ret.z = coords.z;
-	return ret;
-}
-
 void DrawEngineCommon::NotifyConfigChanged() {
 	if (decJitCache_)
 		decJitCache_->Clear();
@@ -128,19 +101,9 @@ void DrawEngineCommon::NotifyConfigChanged() {
 		delete decoder;
 	});
 	decoderMap_.Clear();
-	ClearTrackedVertexArrays();
 
 	useHWTransform_ = g_Config.bHardwareTransform;
 	useHWTessellation_ = UpdateUseHWTessellation(g_Config.bHardwareTessellation);
-	decOptions_.applySkinInDecode = g_Config.bSoftwareSkinning;
-}
-
-u32 DrawEngineCommon::NormalizeVertices(u8 *outPtr, u8 *bufPtr, const u8 *inPtr, int lowerBound, int upperBound, u32 vertType, int *vertexSize) {
-	const u32 vertTypeID = GetVertTypeID(vertType, gstate.getUVGenMode(), decOptions_.applySkinInDecode);
-	VertexDecoder *dec = GetVertexDecoder(vertTypeID);
-	if (vertexSize)
-		*vertexSize = dec->VertexSize();
-	return DrawEngineCommon::NormalizeVertices(outPtr, bufPtr, inPtr, dec, lowerBound, upperBound, vertType);
 }
 
 void DrawEngineCommon::DispatchSubmitImm(GEPrimitiveType prim, TransformedVertex *buffer, int vertexCount, int cullMode, bool continuation) {
@@ -170,10 +133,10 @@ void DrawEngineCommon::DispatchSubmitImm(GEPrimitiveType prim, TransformedVertex
 	// TODO: Handle fog and secondary color somehow?
 
 	if (gstate.isFogEnabled() && !gstate.isModeThrough()) {
-		WARN_LOG_REPORT_ONCE(geimmfog, G3D, "Imm vertex used fog");
+		WARN_LOG_REPORT_ONCE(geimmfog, Log::G3D, "Imm vertex used fog");
 	}
 	if (color1Used != 0 && gstate.isUsingSecondaryColor() && !gstate.isModeThrough()) {
-		WARN_LOG_REPORT_ONCE(geimmcolor1, G3D, "Imm vertex used secondary color");
+		WARN_LOG_REPORT_ONCE(geimmcolor1, Log::G3D, "Imm vertex used secondary color");
 	}
 
 	bool prevThrough = gstate.isModeThrough();
@@ -184,11 +147,12 @@ void DrawEngineCommon::DispatchSubmitImm(GEPrimitiveType prim, TransformedVertex
 	}
 
 	int bytesRead;
-	uint32_t vertTypeID = GetVertTypeID(vtype, 0, decOptions_.applySkinInDecode);
+	uint32_t vertTypeID = GetVertTypeID(vtype, 0, applySkinInDecode_);
 
 	bool clockwise = !gstate.isCullEnabled() || gstate.getCullMode() == cullMode;
-	SubmitPrim(&temp[0], nullptr, prim, vertexCount, vertTypeID, clockwise, &bytesRead);
-	DispatchFlush();
+	VertexDecoder *dec = GetVertexDecoder(vertTypeID);
+	SubmitPrim(&temp[0], nullptr, prim, vertexCount, dec, vertTypeID, clockwise, &bytesRead);
+	Flush();
 
 	if (!prevThrough) {
 		gstate.vertType &= ~GE_VTYPE_THROUGH;
@@ -252,8 +216,7 @@ void DrawEngineCommon::UpdatePlanes() {
 // * Compute min/max of the verts, and then compute a bounding sphere and check that against the planes.
 //   - Less accurate, but..
 //   - Only requires six plane evaluations then.
-
-bool DrawEngineCommon::TestBoundingBox(const void *vdata, const void *inds, int vertexCount, u32 vertType) {
+bool DrawEngineCommon::TestBoundingBox(const void *vdata, const void *inds, int vertexCount, const VertexDecoder *dec, u32 vertType) {
 	// Grab temp buffer space from large offsets in decoded_. Not exactly safe for large draws.
 	if (vertexCount > 1024) {
 		return true;
@@ -302,11 +265,8 @@ bool DrawEngineCommon::TestBoundingBox(const void *vdata, const void *inds, int
 			}
 			// TODO: Avoid normalization if just plain skinning.
 			// Force software skinning.
-			bool wasApplyingSkinInDecode = decOptions_.applySkinInDecode;
-			decOptions_.applySkinInDecode = true;
-			NormalizeVertices((u8 *)corners, temp_buffer, (const u8 *)vdata, indexLowerBound, indexUpperBound, vertType);
-			decOptions_.applySkinInDecode = wasApplyingSkinInDecode;
-
+			const u32 vertTypeID = GetVertTypeID(vertType, gstate.getUVGenMode(), true);
+			::NormalizeVertices(corners, temp_buffer, (const u8 *)vdata, indexLowerBound, indexUpperBound, dec, vertType);
 			IndexConverter conv(vertType, inds);
 			for (int i = 0; i < vertexCount; i++) {
 				verts[i * 3] = corners[conv(i)].pos.x;
@@ -315,7 +275,6 @@ bool DrawEngineCommon::TestBoundingBox(const void *vdata, const void *inds, int
 			}
 		} else {
 			// Simple, most common case.
-			VertexDecoder *dec = GetVertexDecoder(vertType);
 			int stride = dec->VertexSize();
 			int offset = dec->posoff;
 			switch (vertType & GE_VTYPE_POS_MASK) {
@@ -392,7 +351,11 @@ bool DrawEngineCommon::TestBoundingBox(const void *vdata, const void *inds, int
 }
 
 // NOTE: This doesn't handle through-mode, indexing, morph, or skinning.
-bool DrawEngineCommon::TestBoundingBoxFast(const void *vdata, int vertexCount, u32 vertType) {
+// TODO: For high vertex counts, we should just take the min/max of all the verts, and test the resulting six cube
+// corners. That way we can cull more draws quite cheaply.
+// We could take the min/max during the regular vertex decode, and just skip the draw call if it's trivially culled.
+// This would help games like Midnight Club (that one does a lot of out-of-bounds drawing) immensely.
+bool DrawEngineCommon::TestBoundingBoxFast(const void *vdata, int vertexCount, const VertexDecoder *dec, u32 vertType) {
 	SimpleVertex *corners = (SimpleVertex *)(decoded_ + 65536 * 12);
 	float *verts = (float *)(decoded_ + 65536 * 18);
 
@@ -415,7 +378,6 @@ bool DrawEngineCommon::TestBoundingBoxFast(const void *vdata, int vertexCount, u
 		return true;
 
 	// Simple, most common case.
-	VertexDecoder *dec = GetVertexDecoder(vertType);
 	int stride = dec->VertexSize();
 	int offset = dec->posoff;
 	int vertStride = 3;
@@ -564,227 +526,64 @@ bool DrawEngineCommon::TestBoundingBoxFast(const void *vdata, int vertexCount, u
 	return true;
 }
 
-// TODO: This probably is not the best interface.
-bool DrawEngineCommon::GetCurrentSimpleVertices(int count, std::vector<GPUDebugVertex> &vertices, std::vector<u16> &indices) {
-	// This is always for the current vertices.
-	u16 indexLowerBound = 0;
-	u16 indexUpperBound = count - 1;
-
-	if (!Memory::IsValidAddress(gstate_c.vertexAddr) || count == 0)
-		return false;
-
-	bool savedVertexFullAlpha = gstate_c.vertexFullAlpha;
-
-	if ((gstate.vertType & GE_VTYPE_IDX_MASK) != GE_VTYPE_IDX_NONE) {
-		const u8 *inds = Memory::GetPointer(gstate_c.indexAddr);
-		const u16_le *inds16 = (const u16_le *)inds;
-		const u32_le *inds32 = (const u32_le *)inds;
-
-		if (inds) {
-			GetIndexBounds(inds, count, gstate.vertType, &indexLowerBound, &indexUpperBound);
-			indices.resize(count);
-			switch (gstate.vertType & GE_VTYPE_IDX_MASK) {
-			case GE_VTYPE_IDX_8BIT:
-				for (int i = 0; i < count; ++i) {
-					indices[i] = inds[i];
-				}
-				break;
-			case GE_VTYPE_IDX_16BIT:
-				for (int i = 0; i < count; ++i) {
-					indices[i] = inds16[i];
-				}
-				break;
-			case GE_VTYPE_IDX_32BIT:
-				WARN_LOG_REPORT_ONCE(simpleIndexes32, G3D, "SimpleVertices: Decoding 32-bit indexes");
-				for (int i = 0; i < count; ++i) {
-					// These aren't documented and should be rare.  Let's bounds check each one.
-					if (inds32[i] != (u16)inds32[i]) {
-						ERROR_LOG_REPORT_ONCE(simpleIndexes32Bounds, G3D, "SimpleVertices: Index outside 16-bit range");
-					}
-					indices[i] = (u16)inds32[i];
-				}
-				break;
-			}
-		} else {
-			indices.clear();
-		}
-	} else {
-		indices.clear();
+// 2D bounding box test against scissor. No indexing yet.
+// Only supports non-indexed draws with float positions.
+bool DrawEngineCommon::TestBoundingBoxThrough(const void *vdata, int vertexCount, const VertexDecoder *dec, u32 vertType, int *bytesRead) {
+	// Grab temp buffer space from large offsets in decoded_. Not exactly safe for large draws.
+	if (vertexCount > 16) {
+		return true;
 	}
 
-	static std::vector<u32> temp_buffer;
-	static std::vector<SimpleVertex> simpleVertices;
-	temp_buffer.resize(std::max((int)indexUpperBound, 8192) * 128 / sizeof(u32));
-	simpleVertices.resize(indexUpperBound + 1);
-	NormalizeVertices((u8 *)(&simpleVertices[0]), (u8 *)(&temp_buffer[0]), Memory::GetPointerUnchecked(gstate_c.vertexAddr), indexLowerBound, indexUpperBound, gstate.vertType);
+	// Although this may lead to drawing that shouldn't happen, the viewport is more complex on VR.
+	// Let's always say objects are within bounds.
+	if (gstate_c.Use(GPU_USE_VIRTUAL_REALITY))
+		return true;
 
-	float world[16];
-	float view[16];
-	float worldview[16];
-	float worldviewproj[16];
-	ConvertMatrix4x3To4x4(world, gstate.worldMatrix);
-	ConvertMatrix4x3To4x4(view, gstate.viewMatrix);
-	Matrix4ByMatrix4(worldview, world, view);
-	Matrix4ByMatrix4(worldviewproj, worldview, gstate.projMatrix);
+	const int stride = dec->VertexSize();
+	const int posOffset = dec->posoff;
 
-	vertices.resize(indexUpperBound + 1);
-	uint32_t vertType = gstate.vertType;
-	for (int i = indexLowerBound; i <= indexUpperBound; ++i) {
-		const SimpleVertex &vert = simpleVertices[i];
+	*bytesRead = stride * vertexCount;
 
-		if ((vertType & GE_VTYPE_THROUGH) != 0) {
-			if (vertType & GE_VTYPE_TC_MASK) {
-				vertices[i].u = vert.uv[0];
-				vertices[i].v = vert.uv[1];
-			} else {
-				vertices[i].u = 0.0f;
-				vertices[i].v = 0.0f;
-			}
-			vertices[i].x = vert.pos.x;
-			vertices[i].y = vert.pos.y;
-			vertices[i].z = vert.pos.z;
-			if (vertType & GE_VTYPE_COL_MASK) {
-				memcpy(vertices[i].c, vert.color, sizeof(vertices[i].c));
-			} else {
-				memset(vertices[i].c, 0, sizeof(vertices[i].c));
-			}
-			vertices[i].nx = 0;  // No meaningful normals in through mode
-			vertices[i].ny = 0;
-			vertices[i].nz = 1.0f;
-		} else {
-			float clipPos[4];
-			Vec3ByMatrix44(clipPos, vert.pos.AsArray(), worldviewproj);
-			Vec3f screenPos = ClipToScreen(clipPos);
-			Vec3f drawPos = ScreenToDrawing(screenPos);
+	bool allOutsideLeft = true;
+	bool allOutsideTop = true;
+	bool allOutsideRight = true;
+	bool allOutsideBottom = true;
+	const float left = gstate.getScissorX1();
+	const float top = gstate.getScissorY1();
+	const float right = gstate.getScissorX2();
+	const float bottom = gstate.getScissorY2();
 
-			if (vertType & GE_VTYPE_TC_MASK) {
-				vertices[i].u = vert.uv[0] * (float)gstate.getTextureWidth(0);
-				vertices[i].v = vert.uv[1] * (float)gstate.getTextureHeight(0);
-			} else {
-				vertices[i].u = 0.0f;
-				vertices[i].v = 0.0f;
+	switch (vertType & GE_VTYPE_POS_MASK) {
+	case GE_VTYPE_POS_FLOAT:
+	{
+		// TODO: This can be SIMD'd, with some trickery.
+		for (int i = 0; i < vertexCount; i++) {
+			const float *pos = (const float*)((const u8 *)vdata + stride * i + posOffset);
+			const float x = pos[0];
+			const float y = pos[1];
+			if (x >= left) {
+				allOutsideLeft = false;
 			}
-			// Should really have separate coordinates for before and after transform.
-			vertices[i].x = drawPos.x;
-			vertices[i].y = drawPos.y;
-			vertices[i].z = drawPos.z;
-			if (vertType & GE_VTYPE_COL_MASK) {
-				memcpy(vertices[i].c, vert.color, sizeof(vertices[i].c));
-			} else {
-				memset(vertices[i].c, 0, sizeof(vertices[i].c));
+			if (x <= right + 1) {
+				allOutsideRight = false;
+			}
+			if (y >= top) {
+				allOutsideTop = false;
+			}
+			if (y <= bottom + 1) {
+				allOutsideBottom = false;
 			}
-			vertices[i].nx = vert.nrm.x;
-			vertices[i].ny = vert.nrm.y;
-			vertices[i].nz = vert.nrm.z;
 		}
+		if (allOutsideLeft || allOutsideTop || allOutsideRight || allOutsideBottom) {
+			return false;
+		}
+		return true;
 	}
-
-	gstate_c.vertexFullAlpha = savedVertexFullAlpha;
-
-	return true;
-}
-
-// This normalizes a set of vertices in any format to SimpleVertex format, by processing away morphing AND skinning.
-// The rest of the transform pipeline like lighting will go as normal, either hardware or software.
-// The implementation is initially a bit inefficient but shouldn't be a big deal.
-// An intermediate buffer of not-easy-to-predict size is stored at bufPtr.
-u32 DrawEngineCommon::NormalizeVertices(u8 *outPtr, u8 *bufPtr, const u8 *inPtr, VertexDecoder *dec, int lowerBound, int upperBound, u32 vertType) {
-	// First, decode the vertices into a GPU compatible format. This step can be eliminated but will need a separate
-	// implementation of the vertex decoder.
-	dec->DecodeVerts(bufPtr, inPtr, &gstate_c.uv, lowerBound, upperBound);
-
-	// OK, morphing eliminated but bones still remain to be taken care of.
-	// Let's do a partial software transform where we only do skinning.
-
-	VertexReader reader(bufPtr, dec->GetDecVtxFmt(), vertType);
-
-	SimpleVertex *sverts = (SimpleVertex *)outPtr;
-
-	const u8 defaultColor[4] = {
-		(u8)gstate.getMaterialAmbientR(),
-		(u8)gstate.getMaterialAmbientG(),
-		(u8)gstate.getMaterialAmbientB(),
-		(u8)gstate.getMaterialAmbientA(),
-	};
-
-	// Let's have two separate loops, one for non skinning and one for skinning.
-	if (!dec->skinInDecode && (vertType & GE_VTYPE_WEIGHT_MASK) != GE_VTYPE_WEIGHT_NONE) {
-		int numBoneWeights = vertTypeGetNumBoneWeights(vertType);
-		for (int i = lowerBound; i <= upperBound; i++) {
-			reader.Goto(i - lowerBound);
-			SimpleVertex &sv = sverts[i];
-			if (vertType & GE_VTYPE_TC_MASK) {
-				reader.ReadUV(sv.uv);
-			}
-
-			if (vertType & GE_VTYPE_COL_MASK) {
-				sv.color_32 = reader.ReadColor0_8888();
-			} else {
-				memcpy(sv.color, defaultColor, 4);
-			}
-
-			float nrm[3], pos[3];
-			float bnrm[3], bpos[3];
-
-			if (vertType & GE_VTYPE_NRM_MASK) {
-				// Normals are generated during tessellation anyway, not sure if any need to supply
-				reader.ReadNrm(nrm);
-			} else {
-				nrm[0] = 0;
-				nrm[1] = 0;
-				nrm[2] = 1.0f;
-			}
-			reader.ReadPos(pos);
-
-			// Apply skinning transform directly
-			float weights[8];
-			reader.ReadWeights(weights);
-			// Skinning
-			Vec3Packedf psum(0, 0, 0);
-			Vec3Packedf nsum(0, 0, 0);
-			for (int w = 0; w < numBoneWeights; w++) {
-				if (weights[w] != 0.0f) {
-					Vec3ByMatrix43(bpos, pos, gstate.boneMatrix + w * 12);
-					Vec3Packedf tpos(bpos);
-					psum += tpos * weights[w];
-
-					Norm3ByMatrix43(bnrm, nrm, gstate.boneMatrix + w * 12);
-					Vec3Packedf tnorm(bnrm);
-					nsum += tnorm * weights[w];
-				}
-			}
-			sv.pos = psum;
-			sv.nrm = nsum;
-		}
-	} else {
-		for (int i = lowerBound; i <= upperBound; i++) {
-			reader.Goto(i - lowerBound);
-			SimpleVertex &sv = sverts[i];
-			if (vertType & GE_VTYPE_TC_MASK) {
-				reader.ReadUV(sv.uv);
-			} else {
-				sv.uv[0] = 0.0f;  // This will get filled in during tessellation
-				sv.uv[1] = 0.0f;
-			}
-			if (vertType & GE_VTYPE_COL_MASK) {
-				sv.color_32 = reader.ReadColor0_8888();
-			} else {
-				memcpy(sv.color, defaultColor, 4);
-			}
-			if (vertType & GE_VTYPE_NRM_MASK) {
-				// Normals are generated during tessellation anyway, not sure if any need to supply
-				reader.ReadNrm((float *)&sv.nrm);
-			} else {
-				sv.nrm.x = 0.0f;
-				sv.nrm.y = 0.0f;
-				sv.nrm.z = 1.0f;
-			}
-			reader.ReadPos((float *)&sv.pos);
-		}
+	default:
+		// Shouldn't end up here with the checks outside this function.
+		_dbg_assert_(false);
+		return true;
 	}
-
-	// Okay, there we are! Return the new type (but keep the index bits)
-	return GE_VTYPE_TC_FLOAT | GE_VTYPE_COL_8888 | GE_VTYPE_NRM_FLOAT | GE_VTYPE_POS_FLOAT | (vertType & (GE_VTYPE_IDX_MASK | GE_VTYPE_THROUGH));
 }
 
 void DrawEngineCommon::ApplyFramebufferRead(FBOTexState *fboTexState) {
@@ -794,7 +593,6 @@ void DrawEngineCommon::ApplyFramebufferRead(FBOTexState *fboTexState) {
 		gpuStats.numCopiesForShaderBlend++;
 		*fboTexState = FBO_TEX_COPY_BIND_TEX;
 	}
-
 	gstate_c.Dirty(DIRTY_SHADERBLEND);
 }
 
@@ -806,7 +604,9 @@ int DrawEngineCommon::ComputeNumVertsToDecode() const {
 	return sum;
 }
 
-int DrawEngineCommon::ExtendNonIndexedPrim(const uint32_t *cmd, const uint32_t *stall, u32 vertTypeID, bool clockwise, int *bytesRead, bool isTriangle) {
+// Takes a list of consecutive PRIM opcodes, and extends the current draw call to include them.
+// This is just a performance optimization.
+int DrawEngineCommon::ExtendNonIndexedPrim(const uint32_t *cmd, const uint32_t *stall, const VertexDecoder *dec, u32 vertTypeID, bool clockwise, int *bytesRead, bool isTriangle) {
 	const uint32_t *start = cmd;
 	int prevDrawVerts = numDrawVerts_ - 1;
 	DeferredVerts &dv = drawVerts_[prevDrawVerts];
@@ -849,13 +649,13 @@ int DrawEngineCommon::ExtendNonIndexedPrim(const uint32_t *cmd, const uint32_t *
 	dv.vertexCount = offset;
 	dv.indexUpperBound = dv.vertexCount - 1;
 	vertexCountInDrawCalls_ += totalCount;
-	*bytesRead = totalCount * dec_->VertexSize();
+	*bytesRead = totalCount * dec->VertexSize();
 	return cmd - start;
 }
 
-void DrawEngineCommon::SkipPrim(GEPrimitiveType prim, int vertexCount, u32 vertTypeID, int *bytesRead) {
+void DrawEngineCommon::SkipPrim(GEPrimitiveType prim, int vertexCount, const VertexDecoder *dec, u32 vertTypeID, int *bytesRead) {
 	if (!indexGen.PrimCompatible(prevPrim_, prim)) {
-		DispatchFlush();
+		Flush();
 	}
 
 	// This isn't exactly right, if we flushed, since prims can straddle previous calls.
@@ -869,19 +669,13 @@ void DrawEngineCommon::SkipPrim(GEPrimitiveType prim, int vertexCount, u32 vertT
 		prevPrim_ = prim;
 	}
 
-	// If vtype has changed, setup the vertex decoder.
-	if (vertTypeID != lastVType_ || !dec_) {
-		dec_ = GetVertexDecoder(vertTypeID);
-		lastVType_ = vertTypeID;
-	}
-
-	*bytesRead = vertexCount * dec_->VertexSize();
+	*bytesRead = vertexCount * dec->VertexSize();
 }
 
 // vertTypeID is the vertex type but with the UVGen mode smashed into the top bits.
-bool DrawEngineCommon::SubmitPrim(const void *verts, const void *inds, GEPrimitiveType prim, int vertexCount, u32 vertTypeID, bool clockwise, int *bytesRead) {
+bool DrawEngineCommon::SubmitPrim(const void *verts, const void *inds, GEPrimitiveType prim, int vertexCount, const VertexDecoder *dec, u32 vertTypeID, bool clockwise, int *bytesRead) {
 	if (!indexGen.PrimCompatible(prevPrim_, prim) || numDrawVerts_ >= MAX_DEFERRED_DRAW_VERTS || numDrawInds_ >= MAX_DEFERRED_DRAW_INDS || vertexCountInDrawCalls_ + vertexCount > VERTEX_BUFFER_MAX) {
-		DispatchFlush();
+		Flush();
 	}
 	_dbg_assert_(numDrawVerts_ < MAX_DEFERRED_DRAW_VERTS);
 	_dbg_assert_(numDrawInds_ < MAX_DEFERRED_DRAW_INDS);
@@ -899,8 +693,11 @@ bool DrawEngineCommon::SubmitPrim(const void *verts, const void *inds, GEPrimiti
 
 	// If vtype has changed, setup the vertex decoder. Don't need to nullcheck dec_ since we set lastVType_ to an invalid value whenever we null it.
 	if (vertTypeID != lastVType_) {
-		dec_ = GetVertexDecoder(vertTypeID);
+		dec_ = dec;
+		_dbg_assert_(dec->VertexType() == vertTypeID);
 		lastVType_ = vertTypeID;
+	} else {
+		_dbg_assert_(dec_->VertexType() == lastVType_);
 	}
 
 	*bytesRead = vertexCount * dec_->VertexSize();
@@ -916,7 +713,7 @@ bool DrawEngineCommon::SubmitPrim(const void *verts, const void *inds, GEPrimiti
 		}
 	}
 
-	bool applySkin = (vertTypeID & GE_VTYPE_WEIGHT_MASK) && decOptions_.applySkinInDecode;
+	bool applySkin = dec_->skinInDecode;
 
 	DeferredInds &di = drawInds_[numDrawInds_++];
 	di.inds = inds;
@@ -964,25 +761,37 @@ bool DrawEngineCommon::SubmitPrim(const void *verts, const void *inds, GEPrimiti
 	if (prim == GE_PRIM_RECTANGLES && (gstate.getTextureAddress(0) & 0x3FFFFFFF) == (gstate.getFrameBufAddress() & 0x3FFFFFFF)) {
 		// This prevents issues with consecutive self-renders in Ridge Racer.
 		gstate_c.Dirty(DIRTY_TEXTURE_PARAMS);
-		DispatchFlush();
+		Flush();
 	}
 	return true;
 }
 
-void DrawEngineCommon::DecodeVerts(u8 *dest) {
-	// Note that this should be able to continue a partial decode - we don't necessarily start from zero here (although we do most of the time).
+void DrawEngineCommon::BeginFrame() {
+	applySkinInDecode_ = g_Config.bSoftwareSkinning;
+}
 
+void DrawEngineCommon::DecodeVerts(const VertexDecoder *dec, u8 *dest) {
+	if (!numDrawVerts_) {
+		return;
+	}
+	// Note that this should be able to continue a partial decode - we don't necessarily start from zero here (although we do most of the time).
 	int i = decodeVertsCounter_;
-	int stride = (int)dec_->GetDecVtxFmt().stride;
+	int stride = (int)dec->GetDecVtxFmt().stride;
 	for (; i < numDrawVerts_; i++) {
-		DeferredVerts &dv = drawVerts_[i];
+		const DeferredVerts &dv = drawVerts_[i];
 
 		int indexLowerBound = dv.indexLowerBound;
 		drawVertexOffsets_[i] = numDecodedVerts_ - indexLowerBound;
 
 		int indexUpperBound = dv.indexUpperBound;
+
+		if (indexUpperBound + 1 - indexLowerBound + numDecodedVerts_ >= VERTEX_BUFFER_MAX) {
+			// Hit our limit! Stop decoding in this draw.
+			break;
+		}
+
 		// Decode the verts (and at the same time apply morphing/skinning). Simple.
-		dec_->DecodeVerts(dest + numDecodedVerts_ * stride, dv.verts, &dv.uvScale, indexLowerBound, indexUpperBound);
+		dec->DecodeVerts(dest + numDecodedVerts_ * stride, dv.verts, &dv.uvScale, indexLowerBound, indexUpperBound);
 		numDecodedVerts_ += indexUpperBound - indexLowerBound + 1;
 	}
 	decodeVertsCounter_ = i;
@@ -1019,13 +828,13 @@ int DrawEngineCommon::DecodeInds() {
 	return indexGen.VertexCount();
 }
 
-bool DrawEngineCommon::CanUseHardwareTransform(int prim) {
+bool DrawEngineCommon::CanUseHardwareTransform(int prim) const {
 	if (!useHWTransform_)
 		return false;
 	return !gstate.isModeThrough() && prim != GE_PRIM_RECTANGLES && prim > GE_PRIM_LINE_STRIP;
 }
 
-bool DrawEngineCommon::CanUseHardwareTessellation(GEPatchPrimType prim) {
+bool DrawEngineCommon::CanUseHardwareTessellation(GEPatchPrimType prim) const {
 	if (useHWTessellation_) {
 		return CanUseHardwareTransform(PatchPrimToPrim(prim));
 	}
@@ -1083,3 +892,273 @@ bool DrawEngineCommon::DescribeCodePtr(const u8 *ptr, std::string &name) const {
 		return false;
 	}
 }
+
+enum {
+	DEPTH_TRANSFORMED_MAX_VERTS = VERTEX_BUFFER_MAX,
+	DEPTH_TRANSFORMED_BYTES = DEPTH_TRANSFORMED_MAX_VERTS * 4 * sizeof(float),
+	DEPTH_SCREENVERTS_COMPONENT_COUNT = VERTEX_BUFFER_MAX,
+	DEPTH_SCREENVERTS_COMPONENT_BYTES = DEPTH_SCREENVERTS_COMPONENT_COUNT * sizeof(int) + 384,
+	DEPTH_SCREENVERTS_TOTAL_BYTES = DEPTH_SCREENVERTS_COMPONENT_BYTES * 3,
+	DEPTH_INDEXBUFFER_BYTES = DEPTH_TRANSFORMED_MAX_VERTS * 3 * sizeof(uint16_t),  // hmmm
+};
+
+// We process vertices for depth rendering in several stages:
+// First, we transform and collect vertices into depthTransformed_ (4-vectors, xyzw).
+// Then, we group and cull the vertices into four-triangle groups, which are placed in
+// depthScreenVerts_, with x, y and z separated into different part of the array.
+// (Alternatively, if drawing rectangles, they're just added linearly).
+// After that, we send these groups out for SIMD setup and rasterization.
+void DrawEngineCommon::InitDepthRaster() {
+	switch ((DepthRasterMode)g_Config.iDepthRasterMode) {
+	case DepthRasterMode::DEFAULT:
+	case DepthRasterMode::LOW_QUALITY:
+		useDepthRaster_ = PSP_CoreParameter().compat.flags().SoftwareRasterDepth;
+		break;
+	case DepthRasterMode::FORCE_ON:
+		useDepthRaster_ = true;
+		break;
+	case DepthRasterMode::OFF:
+		useDepthRaster_ = false;
+	}
+
+	if (useDepthRaster_) {
+		depthDraws_.reserve(256);
+		depthTransformed_ = (float *)AllocateMemoryPages(DEPTH_TRANSFORMED_BYTES, MEM_PROT_READ | MEM_PROT_WRITE);
+		depthScreenVerts_ = (int *)AllocateMemoryPages(DEPTH_SCREENVERTS_TOTAL_BYTES, MEM_PROT_READ | MEM_PROT_WRITE);
+		depthIndices_ = (uint16_t *)AllocateMemoryPages(DEPTH_INDEXBUFFER_BYTES, MEM_PROT_READ | MEM_PROT_WRITE);
+	}
+}
+
+void DrawEngineCommon::ShutdownDepthRaster() {
+	if (depthTransformed_) {
+		FreeMemoryPages(depthTransformed_, DEPTH_TRANSFORMED_BYTES);
+	}
+	if (depthScreenVerts_) {
+		FreeMemoryPages(depthScreenVerts_, DEPTH_SCREENVERTS_TOTAL_BYTES);
+	}
+	if (depthIndices_) {
+		FreeMemoryPages(depthIndices_, DEPTH_INDEXBUFFER_BYTES);
+	}
+}
+
+Mat4F32 ComputeFinalProjMatrix() {
+	const float viewportTranslate[4] = {
+		gstate.getViewportXCenter() - gstate.getOffsetX(),
+		gstate.getViewportYCenter() - gstate.getOffsetY(),
+		gstate.getViewportZCenter(),
+		0.0f,
+	};
+
+	Mat4F32 wv = Mul4x3By4x4(Mat4x3F32(gstate.worldMatrix), Mat4F32::Load4x3(gstate.viewMatrix));
+	Mat4F32 m = Mul4x4By4x4(wv, Mat4F32(gstate.projMatrix));
+	// NOTE: Applying the translation actually works pre-divide, since W is also affected.
+	Vec4F32 scale = Vec4F32::LoadF24x3_One(&gstate.viewportxscale);
+	Vec4F32 translate = Vec4F32::Load(viewportTranslate);
+	TranslateAndScaleInplace(m, scale, translate);
+	return m;
+}
+
+bool DrawEngineCommon::CalculateDepthDraw(DepthDraw *draw, GEPrimitiveType prim, int vertexCount) {
+	switch (prim) {
+	case GE_PRIM_INVALID:
+	case GE_PRIM_KEEP_PREVIOUS:
+	case GE_PRIM_LINES:
+	case GE_PRIM_LINE_STRIP:
+	case GE_PRIM_POINTS:
+		return false;
+	default:
+		break;
+	}
+
+	// Ignore some useless compare modes.
+	switch (gstate.getDepthTestFunction()) {
+	case GE_COMP_ALWAYS:
+		draw->compareMode = ZCompareMode::Always;
+		break;
+	case GE_COMP_LEQUAL:
+	case GE_COMP_LESS:
+		draw->compareMode = ZCompareMode::Less;
+		break;
+	case GE_COMP_GEQUAL:
+	case GE_COMP_GREATER:
+		draw->compareMode = ZCompareMode::Greater;  // Most common
+		break;
+	case GE_COMP_NEVER:
+	case GE_COMP_EQUAL:
+		// These will never have a useful effect in Z-only raster.
+		[[fallthrough]];
+	case GE_COMP_NOTEQUAL:
+		// This is highly unusual, let's just ignore it.
+		[[fallthrough]];
+	default:
+		return false;
+	}
+	if (gstate.isModeClear()) {
+		if (!gstate.isClearModeDepthMask()) {
+			return false;
+		}
+		draw->compareMode = ZCompareMode::Always;
+	} else {
+		// These should have been caught earlier.
+		_dbg_assert_(gstate.isDepthTestEnabled());
+		_dbg_assert_(gstate.isDepthWriteEnabled());
+	}
+
+	if (depthVertexCount_ + vertexCount >= DEPTH_TRANSFORMED_MAX_VERTS) {
+		// Can't add more. We need to flush.
+		return false;
+	}
+
+	draw->depthAddr = gstate.getDepthBufRawAddress() | 0x04000000;
+	draw->depthStride = gstate.DepthBufStride();
+	draw->vertexOffset = depthVertexCount_;
+	draw->indexOffset = depthIndexCount_;
+	draw->vertexCount = vertexCount;
+	draw->cullEnabled = gstate.isCullEnabled();
+	draw->cullMode = gstate.getCullMode();
+	draw->prim = prim;
+	draw->scissor.x1 = gstate.getScissorX1();
+	draw->scissor.y1 = gstate.getScissorY1();
+	draw->scissor.x2 = gstate.getScissorX2();
+	draw->scissor.y2 = gstate.getScissorY2();
+	return true;
+}
+
+void DrawEngineCommon::DepthRasterSubmitRaw(GEPrimitiveType prim, const VertexDecoder *dec, uint32_t vertTypeID, int vertexCount) {
+	if (!gstate.isModeClear() && (!gstate.isDepthTestEnabled() || !gstate.isDepthWriteEnabled())) {
+		return;
+	}
+
+	if (vertTypeID & (GE_VTYPE_WEIGHT_MASK | GE_VTYPE_MORPHCOUNT_MASK)) {
+		return;
+	}
+
+	_dbg_assert_(prim != GE_PRIM_RECTANGLES);
+
+	float worldviewproj[16];
+	ComputeFinalProjMatrix().Store(worldviewproj);
+
+	DepthDraw draw;
+	if (!CalculateDepthDraw(&draw, prim, vertexCount)) {
+		return;
+	}
+
+	TimeCollector collectStat(&gpuStats.msPrepareDepth, coreCollectDebugStats);
+
+	// Decode.
+	int numDecoded = 0;
+	for (int i = 0; i < numDrawVerts_; i++) {
+		const DeferredVerts &dv = drawVerts_[i];
+		if (dv.indexUpperBound + 1 - dv.indexLowerBound + numDecoded >= DEPTH_TRANSFORMED_MAX_VERTS) {
+			// Hit our limit! Stop decoding in this draw.
+			// We should have already broken out in CalculateDepthDraw.
+			break;
+		}
+		// Decode the verts (and at the same time apply morphing/skinning). Simple.
+		DecodeAndTransformForDepthRaster(depthTransformed_ + (draw.vertexOffset + numDecoded) * 4, worldviewproj, dv.verts, dv.indexLowerBound, dv.indexUpperBound, dec, vertTypeID);
+		numDecoded += dv.indexUpperBound - dv.indexLowerBound + 1;
+	}
+
+	// Copy indices.
+	memcpy(depthIndices_ + draw.indexOffset, decIndex_, sizeof(uint16_t) * vertexCount);
+
+	// Commit
+	depthIndexCount_ += vertexCount;
+	depthVertexCount_ += numDecoded;
+
+	if (depthDraws_.empty()) {
+		rasterTimeStart_ = time_now_d();
+	}
+
+	depthDraws_.push_back(draw);
+
+	// FlushQueuedDepth();
+}
+
+void DrawEngineCommon::DepthRasterPredecoded(GEPrimitiveType prim, const void *inVerts, int numDecoded, const VertexDecoder *dec, int vertexCount) {
+	if (!gstate.isModeClear() && (!gstate.isDepthTestEnabled() || !gstate.isDepthWriteEnabled())) {
+		return;
+	}
+
+	DepthDraw draw;
+	if (!CalculateDepthDraw(&draw, prim, vertexCount)) {
+		return;
+	}
+
+	TimeCollector collectStat(&gpuStats.msPrepareDepth, coreCollectDebugStats);
+
+	// Make sure these have already been indexed away.
+	_dbg_assert_(prim != GE_PRIM_TRIANGLE_STRIP && prim != GE_PRIM_TRIANGLE_FAN);
+
+	if (dec->throughmode) {
+		ConvertPredecodedThroughForDepthRaster(depthTransformed_ + 4 * draw.vertexOffset, decoded_, dec, numDecoded);
+	} else {
+		if (dec->VertexType() & (GE_VTYPE_WEIGHT_MASK | GE_VTYPE_MORPHCOUNT_MASK)) {
+			return;
+		}
+		float worldviewproj[16];
+		ComputeFinalProjMatrix().Store(worldviewproj);
+		TransformPredecodedForDepthRaster(depthTransformed_ + 4 * draw.vertexOffset, worldviewproj, decoded_, dec, numDecoded);
+	}
+
+	// Copy indices.
+	memcpy(depthIndices_ + draw.indexOffset, decIndex_, sizeof(uint16_t) * vertexCount);
+
+	// Commit
+	depthIndexCount_ += vertexCount;
+	depthVertexCount_ += numDecoded;
+
+	depthDraws_.push_back(draw);
+
+	if (depthDraws_.empty()) {
+		rasterTimeStart_ = time_now_d();
+	}
+	// FlushQueuedDepth();
+}
+
+void DrawEngineCommon::FlushQueuedDepth() {
+	if (rasterTimeStart_ != 0.0) {
+		gpuStats.msRasterTimeAvailable += time_now_d() - rasterTimeStart_;
+		rasterTimeStart_ = 0.0;
+	}
+
+	const bool collectStats = coreCollectDebugStats;
+	const bool lowQ = g_Config.iDepthRasterMode == (int)DepthRasterMode::LOW_QUALITY;
+
+	for (const auto &draw : depthDraws_) {
+		int *tx = depthScreenVerts_;
+		int *ty = depthScreenVerts_ + DEPTH_SCREENVERTS_COMPONENT_COUNT;
+		float *tz = (float *)(depthScreenVerts_ + DEPTH_SCREENVERTS_COMPONENT_COUNT * 2);
+
+		int outVertCount = 0;
+
+		const float *vertices = depthTransformed_ + 4 * draw.vertexOffset;
+		const uint16_t *indices = depthIndices_ + draw.indexOffset;
+
+		DepthScissor tileScissor = draw.scissor.Tile(0, 1);
+
+		{
+			TimeCollector collectStat(&gpuStats.msCullDepth, collectStats);
+			switch (draw.prim) {
+			case GE_PRIM_RECTANGLES:
+				outVertCount = DepthRasterClipIndexedRectangles(tx, ty, tz, vertices, indices, draw, tileScissor);
+				break;
+			case GE_PRIM_TRIANGLES:
+				outVertCount = DepthRasterClipIndexedTriangles(tx, ty, tz, vertices, indices, draw, tileScissor);
+				break;
+			default:
+				_dbg_assert_(false);
+				break;
+			}
+		}
+		{
+			TimeCollector collectStat(&gpuStats.msRasterizeDepth, collectStats);
+			DepthRasterScreenVerts((uint16_t *)Memory::GetPointerWrite(draw.depthAddr), draw.depthStride, tx, ty, tz, outVertCount, draw, tileScissor, lowQ);
+		}
+	}
+
+	// Reset queue
+	depthIndexCount_ = 0;
+	depthVertexCount_ = 0;
+	depthDraws_.clear();
+}
