@@ -30,9 +30,11 @@
 #include "Common/CommonTypes.h"
 #include "Common/Log.h"
 #include "Common/GPU/thin3d.h"
+#include "Core/Config.h"
+#include "Core/ConfigValues.h"
 #include "GPU/GPU.h"
+#include "GPU/GPUDefinitions.h"
 #include "GPU/ge_constants.h"
-#include "GPU/GPUInterface.h"
 #include "GPU/Common/Draw2D.h"
 
 enum {
@@ -283,6 +285,7 @@ class DrawEngineCommon;
 class PresentationCommon;
 class ShaderManagerCommon;
 class TextureCacheCommon;
+struct DisplayLayoutConfig;
 
 class FramebufferManagerCommon {
 public:
@@ -300,18 +303,19 @@ public:
 	}
 
 	void Init(int msaaLevel);
-	virtual void BeginFrame();
+	virtual void BeginFrame(const DisplayLayoutConfig &config);
 	void SetDisplayFramebuffer(u32 framebuf, u32 stride, GEBufferFormat format);
 	void DestroyFramebuf(VirtualFramebuffer *v);
 
 	VirtualFramebuffer *DoSetRenderFrameBuffer(FramebufferHeuristicParams &params, u32 skipDrawReason);
-	VirtualFramebuffer *SetRenderFrameBuffer(bool framebufChanged, int skipDrawReason) {
+	VirtualFramebuffer *SetRenderFrameBuffer(bool framebufChanged, int skipDrawReason, bool *changed) {
 		// Inlining this part since it's so frequent.
 		if (!framebufChanged && currentRenderVfb_) {
 			currentRenderVfb_->last_frame_render = gpuStats.numFlips;
 			currentRenderVfb_->dirtyAfterDisplay = true;
 			if (!skipDrawReason)
 				currentRenderVfb_->reallyDirtyAfterDisplay = true;
+			*changed = false;
 			return currentRenderVfb_;
 		} else {
 			// This is so that we will be able to drive DoSetRenderFramebuffer with inputs
@@ -321,6 +325,7 @@ public:
 			VirtualFramebuffer *vfb = DoSetRenderFrameBuffer(inputs, skipDrawReason);
 			_dbg_assert_msg_(vfb, "DoSetRenderFramebuffer must return a valid framebuffer.");
 			_dbg_assert_msg_(currentRenderVfb_, "DoSetRenderFramebuffer must set a valid framebuffer.");
+			*changed = true;
 			return vfb;
 		}
 	}
@@ -329,7 +334,7 @@ public:
 	void RebindFramebuffer(const char *tag);
 	std::vector<const VirtualFramebuffer *> GetFramebufferList() const;
 
-	void CopyDisplayToOutput(bool reallyDirty);
+	void CopyDisplayToOutput(const DisplayLayoutConfig &config, bool reallyDirty);
 
 	bool NotifyFramebufferCopy(u32 src, u32 dest, int size, GPUCopyFlag flags, u32 skipDrawReason);
 	void PerformWriteFormattedFromMemory(u32 addr, int size, int width, GEBufferFormat fmt);
@@ -355,9 +360,10 @@ public:
 	void ReadFramebufferToMemory(VirtualFramebuffer *vfb, int x, int y, int w, int h, RasterChannel channel, Draw::ReadbackMode mode);
 
 	void DownloadFramebufferForClut(u32 fb_address, u32 loadBytes);
-	void DrawFramebufferToOutput(const u8 *srcPixels, int srcStride, GEBufferFormat srcPixelFormat);
+	bool DrawFramebufferToOutput(const DisplayLayoutConfig &config, const u8 *srcPixels, int srcStride, GEBufferFormat srcPixelFormat);
 
-	void DrawPixels(VirtualFramebuffer *vfb, int dstX, int dstY, const u8 *srcPixels, GEBufferFormat srcPixelFormat, int srcStride, int width, int height, RasterChannel channel, const char *tag);
+	// TODO: Should split into one that uses config, and one that doesn't.
+	void DrawPixels(const DisplayLayoutConfig *config, VirtualFramebuffer *vfb, int dstX, int dstY, const u8 *srcPixels, GEBufferFormat srcPixelFormat, int srcStride, int width, int height, RasterChannel channel, const char *tag);
 
 	size_t NumVFBs() const { return vfbs_.size(); }
 
@@ -382,6 +388,10 @@ public:
 		return useBufferedRendering_;
 	}
 
+	void ForceUseBufferedRendering(bool buf) {
+		useBufferedRendering_ = true;
+	}
+
 	// TODO: Maybe just include the last depth buffer address in this, too.
 	bool MayIntersectFramebufferColor(u32 start) const {
 		// Clear the cache/kernel bits.
@@ -430,11 +440,11 @@ public:
 	}
 	void SetSafeSize(u16 w, u16 h);
 
-	void NotifyRenderResized(int msaaLevel);
+	void NotifyRenderResized(const DisplayLayoutConfig &config, int msaaLevel);
 	virtual void NotifyDisplayResized();
 	void NotifyConfigChanged();
 
-	void CheckPostShaders();
+	void CheckPostShaders(const DisplayLayoutConfig &config);
 
 	virtual void DestroyAllFBOs();
 
@@ -487,6 +497,15 @@ public:
 
 	bool PresentedThisFrame() const;
 
+	const std::vector<VirtualFramebuffer *> &GetVFBs() const {
+		return vfbs_;
+	}
+
+	// Hack, only needed for non-buffered rendering.
+	const DisplayLayoutConfig &GetDisplayLayoutConfigCopy() const {
+		return displayLayoutConfigCopy_;
+	}
+
 protected:
 	virtual void ReadbackFramebuffer(VirtualFramebuffer *vfb, int x, int y, int w, int h, RasterChannel channel, Draw::ReadbackMode mode);
 	// Used for when a shader is required, such as GLES.
@@ -512,15 +531,15 @@ protected:
 	void EstimateDrawingSize(u32 fb_address, int fb_stride, GEBufferFormat fb_format, int viewport_width, int viewport_height, int region_width, int region_height, int scissor_width, int scissor_height, int &drawing_width, int &drawing_height);
 
 	void NotifyRenderFramebufferCreated(VirtualFramebuffer *vfb);
-	void NotifyRenderFramebufferUpdated(VirtualFramebuffer *vfb);
+	static void NotifyRenderFramebufferUpdated(VirtualFramebuffer *vfb);
 	void NotifyRenderFramebufferSwitched(VirtualFramebuffer *prevVfb, VirtualFramebuffer *vfb, bool isClearingDepth);
 
-	void BlitFramebufferDepth(VirtualFramebuffer *src, VirtualFramebuffer *dst);
+	void BlitFramebufferDepth(VirtualFramebuffer *src, VirtualFramebuffer *dst, bool allowSizeMismatch = false);
 
 	void ResizeFramebufFBO(VirtualFramebuffer *vfb, int w, int h, bool force = false, bool skipCopy = false);
 
-	bool ShouldDownloadFramebufferColor(const VirtualFramebuffer *vfb) const;
-	bool ShouldDownloadFramebufferDepth(const VirtualFramebuffer *vfb) const;
+	static bool ShouldDownloadFramebufferColor(const VirtualFramebuffer *vfb);
+	static bool ShouldDownloadFramebufferDepth(const VirtualFramebuffer *vfb);
 	void DownloadFramebufferOnSwitch(VirtualFramebuffer *vfb);
 
 	bool FindTransferFramebuffer(u32 basePtr, int stride, int x, int y, int w, int h, int bpp, bool destination, BlockTransferRect *rect);
@@ -530,7 +549,7 @@ protected:
 
 	VirtualFramebuffer *CreateRAMFramebuffer(uint32_t fbAddress, int width, int height, int stride, GEBufferFormat format);
 
-	void UpdateFramebufUsage(VirtualFramebuffer *vfb);
+	void UpdateFramebufUsage(VirtualFramebuffer *vfb) const;
 
 	int GetFramebufferLayers() const;
 
@@ -548,6 +567,8 @@ protected:
 		return fbBindSeqCount_++;
 	}
 
+	static SkipGPUReadbackMode GetSkipGPUReadbackMode();
+
 	PresentationCommon *presentation_ = nullptr;
 
 	Draw::DrawContext *draw_ = nullptr;
@@ -646,4 +667,9 @@ protected:
 	// Depth readback helper state
 	u8 *convBuf_ = nullptr;
 	u32 convBufSize_ = 0;
+
+	DisplayLayoutConfig displayLayoutConfigCopy_{};
 };
+
+// Should probably live elsewhere.
+bool GetOutputFramebuffer(Draw::DrawContext *draw, GPUDebugBuffer &buffer);
