@@ -28,7 +28,6 @@
 #include "Common/LogReporting.h"
 #include "GPU/ge_constants.h"
 #include "GPU/Common/ShaderCommon.h"
-#include "GPU/GPUCommon.h"
 #include "GPU/GPUState.h"
 
 #if PPSSPP_ARCH(ARM)
@@ -513,6 +512,7 @@ private:
 	void CompareToJit(const u8 *startPtr, u8 *decodedptr, int count, const UVScale *uvScaleOffset) const;
 };
 
+const char *GetStepFunctionName(StepFunction func);
 
 // A compiled vertex decoder takes the following arguments (C calling convention):
 // u8 *src, u8 *dst, int count
