@@ -21,11 +21,8 @@ public:
 	}
 
 	void ThreadStart() override { renderManager_->ThreadStart(draw_); }
-	bool ThreadFrame() override { return renderManager_->ThreadFrame(); }
+	bool ThreadFrame(bool waitIfEmpty) override { return renderManager_->ThreadFrame(waitIfEmpty); }
 	void ThreadEnd() override { renderManager_->ThreadEnd(); }
-	void StopThread() override {
-		renderManager_->StopThread();
-	}
 
 	GPUCore GetGPUCore() override { return GPUCORE_GLES; }
 	const char *Ident() override { return "OpenGL Core"; }
